@@ -19,14 +19,15 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <assert.h>
 #include <librnd/rnd_config.h>
 #include <librnd/core/event.h>
 #include <librnd/core/error.h>
@@ -34,41 +35,41 @@
 #include <librnd/core/fptr_cast.h>
 
 static const char *rnd_evnames_lib[] = {
-	"pcbev_gui_init",
-	"pcbev_cli_enter",
-	"pcbeb_new_tool",
-	"pcbeb_tool_select_pre",
-	"pcbeb_tool_release",
-	"pcbeb_tool_press",
-	"pcbev_busy",
-	"pcbev_log_append",
-	"pcbev_log_clear",
-	"pcbev_gui_lead_user",
-	"pcbev_gui_draw_overlay_xor",
-	"pcbev_user_input_post",
-	"pcbev_user_input_key",
-	"pcbev_crosshair_move",
-	"pcbev_dad_new_dialog",
-	"pcbev_dad_new_geo",
-	"pcbev_export_session_begin",
-	"pcbev_export_session_end",
-	"pcbev_stroke_start",
-	"pcbev_stroke_record",
-	"pcbev_stroke_finish",
-	"pcbev_board_changed",
-	"pcbev_board_meta_changed",
-	"pcbev_board_fn_changed",
-	"pcbev_save_pre",
-	"pcbev_save_post",
-	"pcbev_load_pre",
-	"pcbev_load_post",
-	"pcbev_menu_changed",
-	"pcbev_gui_batch_timer",
-	"pcbev_mainloop_change",
-	"pcbev_design_meta_changed_lib",
-	"pcbev_dad_new_pane",
-	"pcbev_dad_pane_geo_chg",
-	"pcbev_conf_file_save_post"
+	"rndev_gui_init",
+	"rndev_cli_enter",
+	"rndev_new_tool",
+	"rndev_tool_select_pre",
+	"rndev_tool_release",
+	"rndev_tool_press",
+	"rndev_busy",
+	"rndev_log_append",
+	"rndev_log_clear",
+	"rndev_gui_lead_user",
+	"rndev_gui_draw_overlay_xor",
+	"rndev_user_input_post",
+	"rndev_user_input_key",
+	"rndev_crosshair_move",
+	"rndev_dad_new_dialog",
+	"rndev_dad_new_geo",
+	"rndev_export_session_begin",
+	"rndev_export_session_end",
+	"rndev_stroke_start",
+	"rndev_stroke_record",
+	"rndev_stroke_finish",
+	"rndev_design_set_current",
+	"rndev_design_set_current_intr",
+	"rndev_design_meta_changed",
+	"rndev_design_fn_changed",
+	"rndev_save_pre",
+	"rndev_save_post",
+	"rndev_load_pre",
+	"rndev_load_post",
+	"rndev_menu_changed",
+	"rndev_gui_batch_timer",
+	"rndev_mainloop_change",
+	"rndev_dad_new_pane",
+	"rndev_dad_pane_geo_chg",
+	"rndev_conf_file_save_post"
 };
 
 static const char **rnd_evnames_app = NULL;
@@ -183,7 +184,7 @@ const char *rnd_event_name(rnd_event_id_t ev)
 	return rnd_evnames_lib[ev];
 }
 
-void rnd_event(rnd_hidlib_t *hidlib, rnd_event_id_t ev, const char *fmt, ...)
+void rnd_event(rnd_design_t *hidlib, rnd_event_id_t ev, const char *fmt, ...)
 {
 	va_list ap;
 	rnd_event_arg_t argv[RND_EVENT_MAX_ARG], *a;
