@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016, 2017 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016, 2017, 2024 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,11 +24,24 @@
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
+#ifndef LIBRND_CORE_PATHS_H
+#define LIBRND_CORE_PATHS_H
+
 /* Resolve paths, build paths using template */
 
 #include <genvector/gds_char.h>
 #include <librnd/core/global_typedefs.h>
 
+/* Figure out the canonical name of the executed program
+   and return malloc'ed exec prefix that should be saved in the config
+   bin_dir is the full installation path of the bin dir, e.g. "/usr/local/bin"
+   bin_dir_to_execprefix is relative path between the two, typically ".."
+   Must be called very early in librnd initialization because it sets
+   rnd_w32_* paths (dynamic $PREFIX)
+   Moved from hid to core at 4.3.1 to resolve core->hid dependency */
+char *rnd_exec_prefix(char *argv0, const char *bin_dir, const char *bin_dir_to_execprefix);
+
+
 /* Allocate *out and copy the path from in to out, replacing ~ with conf_core.rc.path.home
    If extra_room is non-zero, allocate this many bytes extra for each slot;
    this leaves some room to append a file name. If quiet is non-zero, suppress
@@ -81,3 +94,13 @@ char *rnd_build_argfn(const char *template, rnd_build_argfn_t *arg);
 
 int rnd_build_argfn_cb(void *ctx, gds_t *s, const char **input);
 
+/* Used when ./configured with --floating-fhs: $PREFIX is not a fixed dir */
+#ifdef RND_WANT_FLOATING_FHS
+extern char *rnd_w32_root;     /* installation prefix; what would be $PREFIX on FHS, e.g. /usr/local */
+extern char *rnd_w32_libdir;   /* on FHS this would be $PREFIX/lib*/
+extern char *rnd_w32_bindir;   /* on FHS this would be $PREFIX/bin - on win32 this also hosts the dlls */
+extern char *rnd_w32_sharedir; /* on FHS this would be $PREFIX/share */
+extern char *rnd_w32_cachedir; /* where to store cache files, e.g. gdk pixbuf loader cache; persistent, but not part of the distribution */
+#endif
+
+#endif
