@@ -4,7 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2011 Andrew Poelstra
- *  Copyright (C) 2016,2019,2022 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016,2019,2022,2023 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include <librnd/rnd_config.h>
 #include <librnd/core/rnd_printf.h>
 #include "actions.h"
+#include "hidlib.h"
 
 const char *rnd_printf_slot[RND_PRINTF_SLOT_max] =
 {
@@ -297,8 +298,9 @@ static human_coord_t human_coord[] = {
 RND_INLINE int try_human_coord(rnd_coord_t coord, const rnd_unit_t *unit, double down_limit, double up_limit, int score_factor, double *value, unsigned int *best, const char **suffix)
 {
 	double v, frac, save;
-	long int digs, zeros;
+	long int digs, zeros, nines;
 	unsigned int score;
+	int round_up = 0;
 
 	/* convert the value to the proposed unit */
 	if (unit->family == RND_UNIT_METRIC)
@@ -317,18 +319,47 @@ RND_INLINE int try_human_coord(rnd_coord_t coord, const rnd_unit_t *unit, double
 	/* count trailing zeros after the decimal point up to 8 digits */
 	frac = v - floor(v);
 	digs = frac * 100000000.0;
-	if (digs != 0)
-		for(zeros = 0; (digs % 10) == 0; zeros++, digs /= 10);
-	else
-		zeros = 8;
 
-	/* score is higher for more zeroes */
+	if (digs != 0) {
+		int in_ze = 1, in_ni = 1;
+
+		/* count continous zeros and nines from the back in parallel */
+		for(zeros = nines = 0; (digs > 0) ; digs /= 10)
+		{
+			int d = (digs % 10), ze = (d == 0), ni = (d == 9);
+			if (ze && in_ze) zeros++;
+			else in_ze = 0;
+			if (ni && in_ni) nines++;
+			else in_ni = 0;
+		}
+	}
+	else {
+		zeros = 8;
+		nines = 0;
+	}
+
+/*	rnd_trace("^^ zeros=%d nines=%d\n", zeros, nines);*/
+
+	/* ending to more nines than zeros */
+	if (nines > zeros) {
+		/* go for the nines and get the converter to round up */
+		zeros = nines;
+		round_up = 1;
+	}
+
+	/* score is higher for more zeroes or nines */
 	score = score_factor + 8 * zeros;
 
 /*	printf("try: %s '%.8f' -> %.8f %d score=%d\n", unit->suffix, v, frac, zeros, score);*/
 
 	/* update the best score */
 	if (score > *best) {
+/*		rnd_trace("save= %.16f %d\n", save, round_up);*/
+		if (round_up) {
+			/* ends in 999999, tweak it just a little bit so it's rounded up */
+			save += 0.0000000001;
+/*			rnd_trace("save= %.16f %d\n", save, round_up);*/
+		}
 		*value = save;
 		*best = score;
 		*suffix = unit->suffix;
@@ -343,7 +374,7 @@ static int CoordsToHumanString(gds_t *dest, rnd_coord_t coord, const gds_t *prin
 	char filemode_buff[128]; /* G_ASCII_DTOSTR_BUF_SIZE */
 	char printf_spec_new_local[256];
 	char *printf_spec_new;
-	int i, retval = -1, trunc0;
+	int i, retval = -1, trunc0, from_app;
 	const char *printf_spec = printf_spec_->array;
 	const char *suffix;
 	double value;
@@ -364,14 +395,21 @@ static int CoordsToHumanString(gds_t *dest, rnd_coord_t coord, const gds_t *prin
 		}
 	}
 
-	for(i = 0; i < NUM_HUMAN_COORD; i++) {
-		int res;
-		/* try the base units first */
-		res = try_human_coord(coord, human_coord[i].base_unit, human_coord[i].down_limit, human_coord[i].up_limit, human_coord[i].score_factor,&value, &best_score, &suffix);
-		if (res < 0)
-			try_human_coord(coord, human_coord[i].down_unit, 0, 0, human_coord[i].score_factor, &value, &best_score, &suffix);
-		else if (res > 0)
-			try_human_coord(coord, human_coord[i].up_unit, 0, 0, human_coord[i].score_factor,&value, &best_score, &suffix);
+	if (rnd_app.human_coord != NULL)
+		from_app = rnd_app.human_coord(coord, &value, &suffix);
+	else
+		from_app = 0;
+
+	if (!from_app) {
+		for(i = 0; i < NUM_HUMAN_COORD; i++) {
+			int res;
+			/* try the base units first */
+			res = try_human_coord(coord, human_coord[i].base_unit, human_coord[i].down_limit, human_coord[i].up_limit, human_coord[i].score_factor,&value, &best_score, &suffix);
+			if (res < 0)
+				try_human_coord(coord, human_coord[i].down_unit, 0, 0, human_coord[i].score_factor, &value, &best_score, &suffix);
+			else if (res > 0)
+				try_human_coord(coord, human_coord[i].up_unit, 0, 0, human_coord[i].score_factor,&value, &best_score, &suffix);
+		}
 	}
 
 	make_printf_spec(printf_spec_new, printf_spec, 8, &trunc0);
