@@ -139,6 +139,7 @@ typedef struct {
 	RND_DAD_DECL_NOINIT(dlg)
 	rnd_hid_attribute_t *end;
 	int wout, wunit, wstick, wglob, valid;
+	unsigned is_dbl:1;
 	char buf[128];
 } spin_unit_t;
 
@@ -168,11 +169,15 @@ static void spin_unit_chg_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute
 		rnd_gui->attr_dlg_set_value(hid_ctx, su->wunit, &hv);
 	}
 
-	rnd_snprintf(su->buf, sizeof(su->buf), "%$m*", unit->suffix, su->end->val.crd);
+	if (su->is_dbl)
+		rnd_snprintf(su->buf, sizeof(su->buf), "%f %s", su->end->val.dbl * unit->scale_factor, unit->suffix);
+	else
+		rnd_snprintf(su->buf, sizeof(su->buf), "%$m*", unit->suffix, su->end->val.crd);
 	hv.str = su->buf;
 	rnd_gui->attr_dlg_set_value(hid_ctx, su->wout, &hv);
 	if (!is_globbing && can_glob) {
 		/* unit changed: disable global, accept the user wants to use this unit */
+		hv.dbl = 0;
 		hv.lng = 0;
 		rnd_gui->attr_dlg_set_value(hid_ctx, su->wglob, &hv);
 	}
@@ -188,6 +193,7 @@ static void spin_unit_dialog(void *spin_hid_ctx, rnd_hid_dad_spin_t *spin, rnd_h
 
 	memset(&ctx, 0, sizeof(ctx));
 	ctx.end = end;
+	ctx.is_dbl = (spin->type == RND_DAD_SPIN_FREQ);
 
 	def_unit = spin->empty_unit == NULL ? rnd_conf.editor.grid_unit : spin->empty_unit;
 
@@ -474,6 +480,39 @@ void rnd_dad_spin_txt_change_cb(void *hid_ctx, void *caller_data, rnd_hid_attrib
 			if (!spin->no_unit_chg && (unit != NULL))
 				spin->unit = unit;
 			break;
+		case RND_DAD_SPIN_FREQ:
+			/* special case: 0 is okay without unit */
+			if (is_str_zero(str->val.str)) {
+				end->val.dbl = 0;
+				spin->last_good_dbl = 0;
+				break;
+			}
+			succ = rnd_get_value_unit(str->val.str, &absolute, 0, &d, &unit);
+			if (succ) {
+				SPIN_CLAMP(d);
+				end->val.dbl = d;
+				spin->last_good_dbl = d;
+			}
+			else {
+				char *dend;
+				d = strtod(str->val.str, &dend);
+				while(isspace(*dend)) dend++;
+				if ((spin->empty_unit != NULL) && (*dend == '\0')) {
+					SPIN_CLAMP(d);
+					unit = spin->empty_unit;
+					d /= unit->scale_factor;
+					end->val.dbl = d;
+					spin->last_good_dbl = d;
+				}
+				else {
+					warn = "Invalid value or unit - result is truncated";
+					end->val.dbl = spin->last_good_dbl;
+				}
+			}
+			if (!spin->no_unit_chg && (unit != NULL))
+				spin->unit = unit;
+			break;
+
 		default: rnd_trace("INTERNAL ERROR: spin_set_num\n");
 	}
 
@@ -598,6 +637,13 @@ void rnd_dad_spin_set_num(rnd_hid_attribute_t *attr, long l, double d, rnd_coord
 			free((char *)str->val.str);
 			str->val.str = gen_str_coord(spin, c, NULL, 0);
 			break;
+		case RND_DAD_SPIN_FREQ:
+			attr->val.dbl = d;
+			spin->last_good_dbl = d;
+			spin->unit = NULL;
+			free((char *)str->val.str);
+			str->val.str = gen_str_coord(spin, d, NULL, 0);
+			break;
 		default: rnd_trace("INTERNAL ERROR: spin_set_num\n");
 	}
 }
