@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -32,7 +32,7 @@
 #include "wt_xpm.h"
 #include "wt_colorbtn.h"
 
-#include <librnd/core/hid_dad.h>
+#include <librnd/hid/hid_dad.h>
 
 #include <librnd/plugins/lib_hid_common/dad_markup.h>
 
@@ -78,7 +78,7 @@ static Widget ltf_progress_create(lesstif_attr_dlg_t *ctx, Widget parent, rnd_ha
 
 /* called back from core (which is called from wt_preview) to get the user
    expose function called */
-static void ltf_preview_expose(rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *e)
+static void ltf_preview_expose(rnd_hid_gc_t gc, rnd_hid_expose_ctx_t *e)
 {
 	rnd_ltf_preview_t *pd = e->draw_data;
 	rnd_hid_attribute_t *attr = pd->attr;
@@ -247,7 +247,7 @@ static Widget ltf_preview_create(lesstif_attr_dlg_t *ctx, Widget parent, rnd_hid
 
 	pd->window = XtWindow(pw);
 	pd->pw = pw;
-	pd->redraw_with_board = !!(attr->hatt_flags & RND_HATF_PRV_BOARD);
+	pd->redraw_with_design = !!(attr->hatt_flags & RND_HATF_PRV_DESIGN);
 
 	XtAddCallback(pw, XmNexposeCallback, (XtCallbackProc)rnd_ltf_preview_callback, (XtPointer)pd);
 	XtAddCallback(pw, XmNresizeCallback, (XtCallbackProc)rnd_ltf_preview_callback, (XtPointer)pd);
