@@ -778,8 +778,7 @@ static void rnd_ltf_draw_pixmap_(rnd_design_t *hidlib, rnd_ltf_pixmap_t *lpm, rn
 				XRenderFreePicture(display, lpm->p_img_scaled);
 			if (lpm->p_mask_scaled != 0)
 				XRenderFreePicture(display, lpm->p_mask_scaled);
-
-			lpm->p_img_scaled = XRenderCreatePicture(display, lpm->img_scaled, XRenderFindVisualFormat(display, DefaultVisual(display, screen)), 0, 0);
+			lpm->p_img_scaled = XRenderCreatePicture(display, lpm->pm_scaled, XRenderFindVisualFormat(display, DefaultVisual(display, screen)), 0, 0);
 			if (lpm->pxm->has_transp)
 				lpm->p_mask_scaled = XRenderCreatePicture(display, lpm->mask_scaled, XRenderFindVisualFormat(display, DefaultVisual(display, screen)), 0, 0);
 			else
@@ -790,8 +789,8 @@ static void rnd_ltf_draw_pixmap_(rnd_design_t *hidlib, rnd_ltf_pixmap_t *lpm, rn
 
 #ifdef RND_HAVE_XRENDER
 	if (use_xrender) {
-		fprintf(stderr, "clip xrender\n");
 		XRenderPictureAttributes pa;
+/*		fprintf(stderr, "clip xrender\n");*/
 		pa.clip_mask = mask_bitmap;
 		XRenderChangePicture(display, lpm->p_img_scaled, CPClipMask, &pa);
 		XRenderComposite(display, PictOpOver, lpm->p_img_scaled, lpm->p_mask_scaled, main_picture, 0, 0, 0, 0, Vx(ox), Vy(ox), w, h);
@@ -2855,7 +2854,7 @@ static void ltf_confchg_fullscreen(rnd_conf_native_t *cfg, int arr_idx, void *us
 
 static rnd_conf_hid_id_t lesstif_conf_id = -1;
 
-static void init_conf_watch(rnd_conf_hid_callbacks_t *cbs, const char *path, void (*func)(rnd_conf_native_t *, int))
+static void init_conf_watch(rnd_conf_hid_callbacks_t *cbs, const char *path, void (*func)(rnd_conf_native_t *, int, void *))
 {
 	rnd_conf_native_t *n = rnd_conf_get_field(path);
 	if (n != NULL) {
@@ -2878,7 +2877,7 @@ static int ltf_clip_set(rnd_hid_t *hid, const char *str)
 {
 	static long cnt = 0;
 	long item_id, data_id;
-	XmString lab = XmStringCreateLocalized(rnd_app.package);
+	XmString lab = XmStringCreateLocalized((char *)rnd_app.package);
 	
 	
 	if (XmClipboardStartCopy(display, window, lab, CurrentTime, 0, NULL, &item_id) != XmClipboardSuccess) {
