@@ -11,10 +11,11 @@
 #include <librnd/core/error.h>
 #include <librnd/core/rnd_printf.h>
 
-#include <librnd/core/hid.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/hid_menu.h>
 #include <librnd/core/hid_cfg.h>
 #include <librnd/core/hid_cfg_action.h>
-#include <librnd/core/hid_cfg_input.h>
+#include <librnd/hid/hid_cfg_input.h>
 #include <librnd/core/conf_hid.h>
 #include "lesstif.h"
 #include <librnd/core/paths.h>
@@ -26,7 +27,7 @@
 
 #include <librnd/plugins/lib_hid_common/menu_helper.h>
 
-extern rnd_hidlib_t *ltf_hidlib;
+extern rnd_design_t *ltf_hidlib;
 
 Widget lesstif_menubar;
 rnd_conf_hid_id_t lesstif_menuconf_id = -1;
@@ -326,7 +327,7 @@ int lesstif_key_event(XKeyEvent * e)
 
 /*	printf("KEY lookup: mod=%x sym=%x/%d\n", mods, sym, slen); */
 TODO("TODO#3: pass on raw and translated keys")
-	slen = rnd_hid_cfg_keys_input2(ltf_hidlib, &lesstif_keymap, mods, sym, sym);
+	slen = rnd_hid_cfg_keys_input(ltf_hidlib, &lesstif_keymap, mods, sym, sym);
 	if (slen <= 0)
 		return 1;
 
@@ -438,7 +439,7 @@ static void set_ins_after(Widget menu, lht_node_t *ins_after)
 	}
 }
 
-static void lesstif_confchg_checkbox(rnd_conf_native_t *cfg, int arr_idx)
+static void lesstif_confchg_checkbox(rnd_conf_native_t *cfg, int arr_idx, void *user_data)
 {
 	lesstif_update_widget_flags(NULL, NULL);
 }
@@ -450,22 +451,6 @@ static void add_res2menu_named(Widget menu, lht_node_t *ins_after, lht_node_t *n
 	menu_data_t *md;
 
 	stdarg_n = 0;
-	v = rnd_hid_cfg_menu_field_str(node, RND_MF_FOREGROUND);
-	if (v != NULL)
-		stdarg_do_color_str(v, XmNforeground);
-
-	v = rnd_hid_cfg_menu_field_str(node, RND_MF_BACKGROUND);
-	if (v != NULL)
-		stdarg_do_color_str(v, XmNbackground);
-
-	v = rnd_hid_cfg_menu_field_str(node, RND_MF_FONT);
-	if (v != NULL) {
-		XFontStruct *fs = XLoadQueryFont(display, v);
-		if (fs) {
-			XmFontList fl = XmFontListCreate(fs, XmSTRING_DEFAULT_CHARSET);
-			stdarg(XmNfontList, fl);
-		}
-	}
 
 	kacc = rnd_hid_cfg_menu_field(node, RND_MF_ACCELERATOR, NULL);
 	if (kacc != NULL) {
@@ -558,10 +543,6 @@ TODO(": remove this call")
 			}
 		}
 
-		v = rnd_hid_cfg_menu_field_str(node, RND_MF_ACTIVE);
-		if (v != NULL)
-			note_widget_flag(md->btn, XmNsensitive, v);
-
 		XtManageChild(md->btn);
 		node->user_data = md;
 	}
