@@ -16,7 +16,8 @@ typedef struct {
 
 	/* Spare: see doc/developer/spare.txt */
 	void (*spare_f1)(void), (*spare_f2)(void);
-	long spare_l1, spare_l2, spare_l3, spare_l4;
+	long enable_pixmap; /* bool: enable exporting pixmap objects */
+	long spare_l2, spare_l3, spare_l4;
 	void *spare_p1, *spare_p2, *spare_p3, *spare_p4;
 	double spare_d1, spare_d2, spare_d3, spare_d4;
 } rnd_eps_t;
