@@ -2,13 +2,13 @@
 #include "compat.h"
 
 /* Update adj limits to match the current zoom level */
-static inline void gtkc_scb_zoom_adjustment(GtkWidget *scrollbar, rnd_coord_t view_size, rnd_coord_t board_size)
+static inline void gtkc_scb_zoom_adjustment(GtkWidget *scrollbar, rnd_coord_t view_size, rnd_coord_t dsg_min, rnd_coord_t dsg_max)
 {
 	GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(scrollbar));
 
-	adj->page_size = MIN(view_size, board_size);
-	adj->lower = -view_size;
-	adj->upper = board_size + adj->page_size;
+	adj->page_size = MIN(view_size, dsg_max - dsg_min);
+	adj->lower = dsg_min - view_size;
+	adj->upper = dsg_max + adj->page_size - dsg_min;
 	adj->step_increment = adj->page_size / 100.0;
 	adj->page_increment = adj->page_size / 10.0;
 	gtk_signal_emit_by_name (GTK_OBJECT(adj), "changed");
