@@ -4,14 +4,14 @@
 #if GTK4_BUG_ON_SCROLLBAR_FIXED
 
 /* Update adj limits to match the current zoom level */
-static inline void gtkc_scb_zoom_adjustment(GtkWidget *scrollb, rnd_coord_t view_size, rnd_coord_t board_size)
+static inline void gtkc_scb_zoom_adjustment(GtkWidget *scrollb, rnd_coord_t view_size, rnd_coord_t dsg_min, rnd_coord_t dsg_max)
 {
 	GtkAdjustment *adj = gtk_scrollbar_get_adjustment(GTK_SCROLLBAR(scrollb));
-	double ps = MIN(view_size, board_size);
+	double ps = MIN(view_size, dsg_max - dsg_min);
 
 	gtk_adjustment_set_page_size(adj, ps);
-	gtk_adjustment_set_lower(adj, -view_size);
-	gtk_adjustment_set_upper(adj, board_size + ps);
+	gtk_adjustment_set_lower(adj, dsg_min - view_size);
+	gtk_adjustment_set_upper(adj, dsg_max - dsg_min + ps);
 
 	gtk_adjustment_set_step_increment(adj, ps / 100.0);
 	gtk_adjustment_set_page_increment(adj, ps / 10.0);
@@ -41,10 +41,10 @@ gtk_adjustment_set_value(adj, 20);
 #include "gtkc_scrollbar.h"
 
 /* Update adj limits to match the current zoom level */
-static inline void gtkc_scb_zoom_adjustment(GtkWidget *scb, rnd_coord_t view_size, rnd_coord_t board_size)
+static inline void gtkc_scb_zoom_adjustment(GtkWidget *scb, rnd_coord_t view_size, rnd_coord_t dsg_min, rnd_coord_t dsg_max)
 {
-	double ps = MIN(view_size, board_size);
-	gtkc_scrollbar_set_range(GTKC_SCROLLBAR(scb), -view_size, board_size + ps, ps);
+	double ps = MIN(view_size, (dsg_max - dsg_min));
+	gtkc_scrollbar_set_range(GTKC_SCROLLBAR(scb), dsg_min-view_size, dsg_max - dsg_min + ps, ps);
 }
 
 #define gtkc_scb_getval(scb)     gtkc_scrollbar_get_val(GTKC_SCROLLBAR(scb))
