@@ -21,8 +21,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
@@ -36,7 +36,7 @@
 #include <librnd/rnd_config.h>
 #include "rnd_gtk.h"
 #include "bu_command.h"
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 
 #include <gdk/gdkkeysyms.h>
 
@@ -229,7 +229,7 @@ char *rnd_gtk_command_entry_get(rnd_gtk_command_t *ctx, const char *prompt, cons
 }
 
 
-void rnd_gtk_handle_user_command(rnd_hidlib_t *hl, rnd_gtk_command_t *ctx, rnd_bool raise)
+void rnd_gtk_handle_user_command(rnd_design_t *hl, rnd_gtk_command_t *ctx, rnd_bool raise)
 {
 	char *command;
 
