@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -63,7 +63,7 @@ static GtkWidget *rnd_gtk_progress_create(attr_dlg_t *ctx, rnd_hid_attribute_t *
 	return prg;
 }
 
-static void rnd_gtka_preview_expose(rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *e)
+static void rnd_gtka_preview_expose(rnd_hid_gc_t gc, rnd_hid_expose_ctx_t *e)
 {
 	rnd_hid_preview_t *prv = e->draw_data;
 	prv->user_expose_cb(prv->attrib, prv, gc, e);
@@ -124,7 +124,7 @@ static GtkWidget *rnd_gtk_preview_create(attr_dlg_t *ctx, rnd_hid_attribute_t *a
 	hp->hid_zoomto_cb = rnd_gtka_preview_zoomto;
 	
 	bparent = frame_scroll(parent, attr->rnd_hatt_flags, &ctx->wltop[j]);
-	prv = rnd_gtk_preview_new(ctx->gctx, ctx->gctx->impl.init_drawing_widget, ctx->gctx->impl.preview_expose, rnd_gtka_preview_expose, rnd_gtka_preview_config, attr->wdata);
+	prv = rnd_gtk_preview_new(ctx->gctx, ctx->gctx->impl.init_drawing_widget, ctx->gctx->impl.preview_expose, rnd_gtka_preview_expose, rnd_gtka_preview_config, attr->wdata, hp->loc_dsg);
 	gtkc_box_pack_append(bparent, prv, TRUE, 0);
 	gtk_widget_set_tooltip_text(prv, attr->help_text);
 	p = (rnd_gtk_preview_t *) prv;
@@ -139,7 +139,7 @@ TODO("TODO make these configurable:")
 	p->y_max = RND_MM_TO_COORD(100);
 	p->w_pixels = RND_MM_TO_COORD(10);
 	p->h_pixels = RND_MM_TO_COORD(10);
-	p->redraw_with_board = !!(attr->hatt_flags & RND_HATF_PRV_BOARD);
+	p->redraw_with_design = !!(attr->hatt_flags & RND_HATF_PRV_DESIGN);
 
 	gtk_widget_set_size_request(prv, hp->min_sizex_px, hp->min_sizey_px);
 	return prv;
