@@ -798,6 +798,43 @@ static int rnd_gtk_attr_dlg_widget_poke_string(void *hid_ctx, GtkWidget *w, int
 	return -1;
 }
 
+static int rnd_gtk_attr_dlg_widget_poke_preview(void *hid_ctx, GtkWidget *w, int idx, int argc, fgw_arg_t argv[])
+{
+	rnd_gtk_preview_t *p = (rnd_gtk_preview_t *)w;
+
+	if ((argv[0].type & FGW_STR) != FGW_STR) return -1;
+
+	switch(argv[0].val.str[0]) {
+		case 'm':
+			if (strcmp(argv[0].val.str, "min_zoom") == 0) {
+				if ((argc < 2) || (fgw_arg_conv(&rnd_fgw, &argv[1], FGW_INT) != 0))
+					return -1;
+				p->view.min_zoom = argv[1].val.nat_int;
+				return 0;
+			}
+			break;
+		case 'y':
+			if (strcmp(argv[0].val.str, "yflip") == 0) { /* needs a redraw on caller side! */
+				if ((argc < 2) || (fgw_arg_conv(&rnd_fgw, &argv[1], FGW_INT) != 0))
+					return -1;
+				p->flip_local = 1;
+				p->view.flip_y = argv[1].val.nat_int;
+				return 0;
+			}
+			break;
+		case 'x':
+			if (strcmp(argv[0].val.str, "xflip") == 0) { /* needs a redraw on caller side! */
+				if ((argc < 2) || (fgw_arg_conv(&rnd_fgw, &argv[1], FGW_INT) != 0))
+					return -1;
+				p->flip_local = 1;
+				p->view.flip_x = argv[1].val.nat_int;
+				return 0;
+			}
+			break;
+	}
+	return -1;
+}
+
 int rnd_gtk_attr_dlg_widget_poke(void *hid_ctx, int idx, int argc, fgw_arg_t argv[])
 {
 	attr_dlg_t *ctx = (attr_dlg_t *)hid_ctx;
@@ -809,6 +846,7 @@ int rnd_gtk_attr_dlg_widget_poke(void *hid_ctx, int idx, int argc, fgw_arg_t arg
 	w = ctx->wl[idx];
 	switch(ctx->attrs[idx].type) {
 		case RND_HATT_STRING: return rnd_gtk_attr_dlg_widget_poke_string(hid_ctx, w, idx, argc, argv);
+		case RND_HATT_PREVIEW: return rnd_gtk_attr_dlg_widget_poke_preview(hid_ctx, w, idx, argc, argv);
 		default: return -1;
 	}
 	return -1;
@@ -1032,14 +1070,18 @@ void rnd_gtk_attr_dlg_free(void *hid_ctx)
 
 void rnd_gtk_attr_dlg_free_all(rnd_gtk_t *gctx)
 {
-	attr_dlg_t *curr, *next;
+	attr_dlg_t *curr, *last = NULL;
 
-	/* don't rely on closing always the first, just in case it fails to unlink */
-	for(curr = gdl_first(&gctx->dad_dialogs); curr != NULL; curr = next) {
-		next = curr->link.next;
+	/* have to rely on closing always the first because it may close further
+	   dialogs so ->link.next is unreliable */
+	while((curr = gdl_first(&gctx->dad_dialogs)) != NULL) {
+		if (curr == last) {
+			fprintf(stderr, "rnd_gtk_attr_dlg_free_all(): failed to force-close dialog\n");
+			break;
+		}
+		last = curr;
 		rnd_gtk_attr_dlg_free(curr);
 	}
-	
 }
 
 void rnd_gtk_attr_dlg_property(void *hid_ctx, rnd_hat_property_t prop, const rnd_hid_attr_val_t *val)
@@ -1092,9 +1134,31 @@ int rnd_gtk_attr_dlg_widget_hide(void *hid_ctx, int idx, rnd_bool hide)
 	if ((idx < 0) || (idx >= ctx->n_attrs))
 		return -1;
 
+	if (ctx->attrs[idx].type == RND_HATT_BEGIN_COMPOUND)
+		return -1;
+
+	if (ctx->attrs[idx].type == RND_HATT_END) {
+		rnd_hid_compound_t *cmp = ctx->attrs[idx].wdata;
+		if ((cmp != NULL) && (cmp->widget_state != NULL))
+			return cmp->widget_hide(&ctx->attrs[idx], ctx, idx, hide);
+		else
+			return -1;
+	}
+
 	return rnd_gtk_attr_dlg_widget_hide_(ctx, idx, hide);
 }
 
+int rnd_gtk_attr_dlg_widget_focus(void *hid_ctx, int idx)
+{
+	attr_dlg_t *ctx = hid_ctx;
+
+	if ((idx < 0) || (idx >= ctx->n_attrs))
+		return -1;
+
+	gtk_widget_grab_focus(ctx->wl[idx]);
+	return 0;
+}
+
 int rnd_gtk_attr_dlg_set_value(void *hid_ctx, int idx, const rnd_hid_attr_val_t *val)
 {
 	attr_dlg_t *ctx = hid_ctx;
