@@ -41,7 +41,7 @@ double rnd_gtk_clamp_zoom(const rnd_gtk_view_t *vw, double coord_per_px)
 
 	min_zoom = vw->min_zoom;
 	if (min_zoom <= 0)
-		min_zoom = 200;
+		min_zoom = rnd_conf.editor.min_zoom;
 
 	/* max zoom is calculated so that zoom * canvas_size * 2 doesn't overflow rnd_coord_t */
 	max_zoom_w = (double)RND_COORD_MAX / (double)vw->canvas_width;
@@ -67,30 +67,32 @@ static void uiz_pan_common(rnd_gtk_view_t *v)
 	rnd_gtk_coords_design2event(v, v->design_x, v->design_y, &event_x, &event_y);
 
 
-	/* Don't pan so far the design is completely off the screen */
-	if (v->use_max_hidlib) {
-		if (LOCALFLIPX(v))
+	if (!rnd_conf.editor.unlimited_pan) {
+		/* Don't pan so far the design is completely off the screen */
+		if (v->use_max_hidlib) {
+			if (LOCALFLIPX(v))
+				v->x0 = MAX(-v->width, v->x0);
+			else
+				v->x0 = MAX(VIEW_HIDLIB(v)->dwg.X1 - v->width, v->x0);
+
+			if (LOCALFLIPY(v))
+				v->y0 = MAX(-v->height, v->y0);
+			else
+				v->y0 = MAX(VIEW_HIDLIB(v)->dwg.Y1 - v->height, v->y0);
+
+			v->x0 = MIN(v->x0, VIEW_HIDLIB(v)->dwg.X2 + (VIEW_HIDLIB(v)->dwg.X2 - VIEW_HIDLIB(v)->dwg.X1));
+			v->y0 = MIN(v->y0, VIEW_HIDLIB(v)->dwg.Y2 + (VIEW_HIDLIB(v)->dwg.Y2 - VIEW_HIDLIB(v)->dwg.Y1));
+		}
+		else {
+			assert(v->max_width > 0);
+			assert(v->max_height > 0);
+
 			v->x0 = MAX(-v->width, v->x0);
-		else
-			v->x0 = MAX(VIEW_HIDLIB(v)->dwg.X1 - v->width, v->x0);
-
-		if (LOCALFLIPY(v))
 			v->y0 = MAX(-v->height, v->y0);
-		else
-			v->y0 = MAX(VIEW_HIDLIB(v)->dwg.Y1 - v->height, v->y0);
 
-		v->x0 = MIN(v->x0, VIEW_HIDLIB(v)->dwg.X2 + (VIEW_HIDLIB(v)->dwg.X2 - VIEW_HIDLIB(v)->dwg.X1));
-		v->y0 = MIN(v->y0, VIEW_HIDLIB(v)->dwg.Y2 + (VIEW_HIDLIB(v)->dwg.Y2 - VIEW_HIDLIB(v)->dwg.Y1));
-	}
-	else {
-		assert(v->max_width > 0);
-		assert(v->max_height > 0);
-
-		v->x0 = MAX(-v->width, v->x0);
-		v->y0 = MAX(-v->height, v->y0);
-
-		v->x0 = MIN(v->x0, v->max_width);
-		v->y0 = MIN(v->y0, v->max_height);
+			v->x0 = MIN(v->x0, v->max_width);
+			v->y0 = MIN(v->y0, v->max_height);
+		}
 	}
 
 	/* Fix up noted event coordinates to match where we clamped. Alternatively
@@ -211,8 +213,10 @@ void rnd_gtk_zoom_view_win(rnd_gtk_view_t *v, rnd_coord_t x1, rnd_coord_t y1, rn
 
 void rnd_gtk_pan_view_abs(rnd_gtk_view_t *v, rnd_coord_t design_x, rnd_coord_t design_y, double widget_x, double widget_y)
 {
+	double ya = (double)SIDE_Y(v, design_y);
+	double yb = (double)widget_y * v->coord_per_px;
 	v->x0 = rnd_round((double)SIDE_X(v, design_x) - (double)widget_x * v->coord_per_px);
-	v->y0 = rnd_round((double)SIDE_Y(v, design_y) - (double)widget_y * v->coord_per_px);
+	v->y0 = rnd_round(ya - yb);
 
 	uiz_pan_common(v);
 }
