@@ -1,4 +1,4 @@
-#include <librnd/core/hid.h>
+#include <librnd/hid/hid.h>
 #include "rnd_gtk.h"
 
 rnd_hidval_t rnd_gtk_add_timer(struct rnd_gtk_s *gctx, void (*func)(rnd_hidval_t user_data), unsigned long milliseconds, rnd_hidval_t user_data);
