@@ -22,8 +22,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
@@ -37,7 +37,7 @@
 
 #include <gtk/gtk.h>
 #include <genlist/gendlist.h>
-#include <librnd/core/hid.h>
+#include <librnd/hid/hid.h>
 #include "ui_zoompan.h"
 #include "rnd_gtk.h"
 
@@ -56,7 +56,7 @@ struct rnd_gtk_preview_class_s {
 
 typedef void (*rnd_gtk_init_drawing_widget_t)(GtkWidget *widget, void *port);
 typedef void (*rnd_gtk_preview_config_t)(rnd_gtk_preview_t *gp, GtkWidget *widget);
-typedef gboolean(*rnd_gtk_preview_expose_t)(GtkWidget *widget, rnd_gtk_expose_t *ev, rnd_hid_expose_t expcall, rnd_hid_expose_ctx_t *ctx);
+typedef gboolean(*rnd_gtk_preview_expose_t)(GtkWidget *widget, rnd_gtk_expose_t *ev, rnd_hid_preview_expose_t expcall, rnd_hid_expose_ctx_t *ctx);
 typedef rnd_bool(*rnd_gtk_preview_mouse_ev_t)(void *widget, void *draw_data, rnd_hid_mouse_ev_t kind, rnd_coord_t x, rnd_coord_t y);
 typedef rnd_bool(*rnd_gtk_preview_key_ev_t)(void *widget, void *draw_data, rnd_bool release, rnd_hid_cfg_mod_t mods, unsigned short int key_raw, unsigned short int key_tr);
 
@@ -88,11 +88,11 @@ struct rnd_gtk_preview_s {
 	rnd_gtk_t *ctx;
 	gtkc_event_xyz_t rs, sc, motion, mpress, mrelease, kpress, krelease, ev_destroy;
 	gdl_elem_t link; /* in the list of all previews in ->ctx->previews */
-	unsigned redraw_with_board:1;
+	unsigned redraw_with_design:1;
 	unsigned redrawing:1;
 	unsigned draw_inited:1; /* used to track new_context() state for opengl backends */
 
-	unsigned flip_global:1;  /* flip status is tied to board's flip; ignored if flip_local is set */
+	unsigned flip_global:1;  /* flip status is tied to design's flip; ignored if flip_local is set */
 	unsigned flip_local:1;   /* local flip enabled on tab; if both local and global flips are off, the preview is permanently on no-flip mode */
 };
 
@@ -101,8 +101,9 @@ GType rnd_gtk_preview_get_type(void);
 /* Queries the natural size of a preview widget */
 void rnd_gtk_preview_get_natsize(rnd_gtk_preview_t *preview, int *width, int *height);
 
-GtkWidget *rnd_gtk_preview_new(rnd_gtk_t *ctx, rnd_gtk_init_drawing_widget_t init_widget,
-	rnd_gtk_preview_expose_t expose, rnd_hid_expose_t dialog_draw, rnd_gtk_preview_config_t config, void *draw_data);
+/* If local_dsg is NULL, preview follows the global design at the moment of
+   exposure (matters for size in flip and initial expose context ->design) */
+GtkWidget *rnd_gtk_preview_new(rnd_gtk_t *ctx, rnd_gtk_init_drawing_widget_t init_widget, rnd_gtk_preview_expose_t expose, rnd_hid_expose_cb_t dialog_draw, rnd_gtk_preview_config_t config, void *draw_data, rnd_design_t *local_dsg);
 
 void rnd_gtk_preview_zoomto(rnd_gtk_preview_t *preview, const rnd_box_t *data_view);
 
