@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2018,2023 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -32,8 +32,8 @@
 #include <ctype.h>
 #include <librnd/core/actions.h>
 #include <librnd/core/compat_misc.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_tree.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_tree.h>
 #include <librnd/core/error.h>
 
 #include "act_dad.h"
@@ -62,7 +62,7 @@ union tmp_u {
 
 typedef struct {
 	RND_DAD_DECL_NOINIT(dlg)
-	rnd_hidlib_t *hidlib;
+	rnd_design_t *hidlib;
 	char *name;
 	const char *row_domain;
 	int level;
@@ -73,7 +73,7 @@ typedef struct {
 
 htsp_t dads;
 
-static int dad_new(rnd_hidlib_t *hl, const char *name)
+static int dad_new(rnd_design_t *hl, const char *name)
 {
 	dad_t *dad;
 
@@ -131,10 +131,10 @@ static void dad_row_free_cb(rnd_hid_attribute_t *attrib, void *hid_ctx, rnd_hid_
 typedef struct {
 	char *act_expose, *act_mouse, *act_free;
 	char *udata;
-	rnd_hidlib_t *hidlib;
+	rnd_design_t *hidlib;
 } dad_prv_t;
 
-static int prv_action(rnd_hidlib_t *hl, const char *actname, rnd_hid_gc_t gc, const char *udata)
+static int prv_action(rnd_design_t *hl, const char *actname, rnd_hid_gc_t gc, const char *udata)
 {
 	fgw_arg_t r = {0};
 	fgw_arg_t args[3];
@@ -159,7 +159,7 @@ static int prv_action(rnd_hidlib_t *hl, const char *actname, rnd_hid_gc_t gc, co
 	return rv;
 }
 
-void dad_prv_expose_cb(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, const rnd_hid_expose_ctx_t *e)
+void dad_prv_expose_cb(rnd_hid_attribute_t *attrib, rnd_hid_preview_t *prv, rnd_hid_gc_t gc, rnd_hid_expose_ctx_t *e)
 {
 	dad_prv_t *ctx = prv->user_ctx;
 	prv_action(ctx->hidlib, ctx->act_expose, gc, ctx->udata);
@@ -259,6 +259,9 @@ const char rnd_acts_dad[] =
 	"dad(dlgname, exists) - returns wheter the named dialog exists (0 or 1)\n"
 	"dad(dlgname, set, widgetID, val) - changes the value of a widget in a running dialog \n"
 	"dad(dlgname, get, widgetID, [unit]) - return the current value of a widget\n"
+	"dad(dlgname, iterate) - runs a global GUI iteration (event dispatch, redraw)\n"
+	"dad(dlgname, raise) - pops up window in front\n"
+	"dad(dlgname, close) - close the dialog (and return 0 from modal run)\n"
 	;
 const char rnd_acth_dad[] = "Manipulate Dynamic Attribute Dialogs";
 fgw_error_t rnd_act_dad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
@@ -271,7 +274,7 @@ fgw_error_t rnd_act_dad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	RND_ACT_CONVARG(2, FGW_STR, dad, cmd = argv[2].val.str);
 
 	if (rnd_strcasecmp(cmd, "new") == 0) {
-		RND_ACT_IRES(dad_new(RND_ACT_HIDLIB, dlgname));
+		RND_ACT_IRES(dad_new(RND_ACT_DESIGN, dlgname));
 		return 0;
 	}
 
@@ -367,6 +370,7 @@ fgw_error_t rnd_act_dad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 				case RND_DAD_SPIN_DOUBLE:  ty = RND_HATT_REAL; break;
 				case RND_DAD_SPIN_FREQ:    ty = RND_HATT_REAL; break;
 				case RND_DAD_SPIN_COORD:   ty = RND_HATT_COORD; break;
+				case RND_DAD_SPIN_UNIT_CRD: ty = RND_HATT_UNIT; break;
 			}
 		}
 
@@ -421,7 +425,7 @@ fgw_error_t rnd_act_dad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		uctx->act_mouse = rnd_concat(prefix, "mouse", NULL);
 		uctx->act_free = rnd_concat(prefix, "free", NULL);
 		uctx->udata = rnd_strdup(suctx);
-		uctx->hidlib = RND_ACT_HIDLIB;
+		uctx->hidlib = RND_ACT_DESIGN;
 		RND_DAD_PREVIEW(dad->dlg, dad_prv_expose_cb, dad_prv_mouse_cb, NULL, dad_prv_free_cb, &vb, sx, sy, uctx);
 	}
 	else if ((rnd_strcasecmp(cmd, "enum") == 0) || (rnd_strcasecmp(cmd, "begin_tabbed") == 0)) {
@@ -507,13 +511,13 @@ fgw_error_t rnd_act_dad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 	}
 	else if (rnd_strcasecmp(cmd, "begin_hpane") == 0) {
 		if (dad->running) goto cant_chg;
-		RND_DAD_BEGIN_HPANE(dad->dlg);
+		RND_DAD_BEGIN_HPANE(dad->dlg, "anon_scripted");
 		dad->level++;
 		rv = RND_DAD_CURRENT(dad->dlg);
 	}
 	else if (rnd_strcasecmp(cmd, "begin_vpane") == 0) {
 		if (dad->running) goto cant_chg;
-		RND_DAD_BEGIN_VPANE(dad->dlg);
+		RND_DAD_BEGIN_VPANE(dad->dlg, "anon_scripted");
 		dad->level++;
 		rv = RND_DAD_CURRENT(dad->dlg);
 	}
@@ -665,7 +669,19 @@ fgw_error_t rnd_act_dad(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 				return FGW_ERR_NOT_FOUND;
 		}
 		return 0;
-	}	else if ((rnd_strcasecmp(cmd, "run") == 0) || (rnd_strcasecmp(cmd, "run_modal") == 0)) {
+	}
+	else if (rnd_strcasecmp(cmd, "iterate") == 0) {
+		rnd_gui->iterate(rnd_gui);
+	}
+	else if (rnd_strcasecmp(cmd, "raise") == 0) {
+		if (rnd_gui->attr_dlg_raise != NULL)
+			rnd_gui->attr_dlg_raise(dad->dlg_hid_ctx);
+	}
+	else if (rnd_strcasecmp(cmd, "close") == 0) {
+		rnd_dad_retovr_t retovr = {0};
+		rnd_hid_dad_close(dad->dlg_hid_ctx, &retovr, 0);
+	}
+	else if ((rnd_strcasecmp(cmd, "run") == 0) || (rnd_strcasecmp(cmd, "run_modal") == 0)) {
 		if (dad->running) goto cant_chg;
 
 		RND_ACT_CONVARG(3, FGW_STR, dad, txt = argv[3].val.str);
