@@ -10,3 +10,6 @@ fgw_error_t rnd_act_PrintDialog(fgw_arg_t *ores, int oargc, fgw_arg_t *oargv);
    actioins, e.g. for exporting a whole project */
 void rnd_dlg_export(const char *title, int exporters, int printers, rnd_design_t *dsg, void *appspec);
 
+#include <librnd/core/event.h>
+void rnd_dialog_export_close(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[]);
+
