@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -31,6 +31,7 @@
 #include <librnd/plugins/lib_hid_common/dlg_pref.h>
 
 static const char *pref_conf_get_val(const lht_node_t *nd, const rnd_conf_native_t *nat, int idx);
+static void dlg_conf_select_node(pref_ctx_t *ctx, const char *path, rnd_conf_native_t *nat, int idx);
 #include "dlg_pref_confedit.c"
 
 /* how many chars per line in conf node description (determines window width vs.
@@ -424,9 +425,8 @@ void rnd_dlg_pref_conf_create(pref_ctx_t *ctx)
 	static const char *hdr_intree[] = {"role", "prio", "policy", "value", NULL};
 
 	RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-	RND_DAD_BEGIN_HPANE(ctx->dlg);
+	RND_DAD_BEGIN_HPANE(ctx->dlg, "conf_left-right");
 		RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-		RND_DAD_SET_PANE_NAME(ctx->dlg, "conf_left-right");
 		ctx->conf.wmainp = RND_DAD_CURRENT(ctx->dlg);
 
 		/* left: tree */
@@ -444,9 +444,8 @@ void rnd_dlg_pref_conf_create(pref_ctx_t *ctx)
 		RND_DAD_END(ctx->dlg);
 
 		/* right: details */
-		RND_DAD_BEGIN_VPANE(ctx->dlg);
+		RND_DAD_BEGIN_VPANE(ctx->dlg, "conf_right_top-bottom");
 			RND_DAD_COMPFLAG(ctx->dlg, RND_HATF_EXPFILL);
-			RND_DAD_SET_PANE_NAME(ctx->dlg, "conf_right_top-bottom");
 
 			/* right/top: conf file */
 			RND_DAD_BEGIN_VBOX(ctx->dlg);
@@ -496,7 +495,13 @@ void rnd_dlg_pref_conf_open(pref_ctx_t *ctx, const char *tabarg)
 }
 
 
-void rnd_dlg_pref_conf_board_changed(pref_ctx_t *ctx)
+void rnd_dlg_pref_conf_design_replaced(pref_ctx_t *ctx)
 {
+	if (ctx->conf.selected_nat != NULL) {
+		/* in a multi-design app each design has its own rnd_conf_fields hash,
+		   we need to look up the same nat in the new design's hash and re-display
+		   that (may have different prio and sources) */
+		ctx->conf.selected_nat = rnd_conf_get_field(ctx->conf.selected_nat->hash_path);
+	}
 	dlg_conf_select_node(ctx, NULL, ctx->conf.selected_nat, ctx->conf.selected_idx);
 }
