@@ -22,8 +22,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -265,12 +265,12 @@ static void direct_draw_lines6(GLfloat *pts, int npts, float red, float green, f
 }
 
 
-static void direct_expose_init(int w, int h, const rnd_color_t *bg_c)
+static void direct_expose_init(int x0, int y0, int w, int h, const rnd_color_t *bg_c)
 {
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-	glViewport(0, 0, w, h);
+	glViewport(x0, y0, w, h);
 
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
