@@ -22,8 +22,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -195,7 +195,7 @@ const char *rnd_script_lang2eng(const char **lang)
 
 	eng = htss_get(&guess_lang_lang2eng, *lang);
 	if (eng == NULL) {
-		/* last resort: maybe *lang is an eng, prefixewd with fungw_ */
+		/* last resort: maybe *lang is an eng, prefixed with fungw_ */
 		char name[RND_PATH_MAX];
 		rnd_snprintf(name, RND_PATH_MAX, "fungw_%s", *lang);
 		eng = htsp_get(&guess_lang_engs, name);
@@ -203,7 +203,7 @@ const char *rnd_script_lang2eng(const char **lang)
 	return eng;
 }
 
-const char *rnd_script_guess_lang(rnd_hidlib_t *hl, const char *fn, int is_filename)
+const char *rnd_script_guess_lang(rnd_design_t *hl, const char *fn, int is_filename)
 {
 	rnd_script_guess_lang_init();
 
