@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -35,12 +35,12 @@
 #include <librnd/core/actions.h>
 #include <librnd/core/plugins.h>
 #include <librnd/core/hid_cfg.h>
-#include <librnd/core/hid_dad.h>
+#include <librnd/hid/hid_dad.h>
 #include <librnd/core/safe_fs.h>
 #include <librnd/core/safe_fs_dir.h>
 #include <librnd/core/compat_fs.h>
 #include <librnd/core/event.h>
-#include <librnd/core/hid_menu.h>
+#include <librnd/hid/hid_menu.h>
 
 #include "script.h"
 
@@ -53,7 +53,7 @@ static const char *lvs_cookie = "live_script";
 
 typedef struct {
 	RND_DAD_DECL_NOINIT(dlg)
-	rnd_hidlib_t *hidlib;
+	rnd_design_t *hidlib;
 	char *name, *longname, *fn;
 	char **langs;
 	char **lang_engines;
@@ -151,7 +151,7 @@ int lvs_list_langs_line_split(pup_list_parse_pup_t *ctx, const char *fname, char
 }
 
 
-static int lvs_list_langs(rnd_hidlib_t *hl, live_script_t *lvs)
+static int lvs_list_langs(rnd_design_t *hl, live_script_t *lvs)
 {
 	const char *paths[2];
 	pup_list_parse_pup_t ctx = {0};
@@ -173,7 +173,7 @@ static int lvs_list_langs(rnd_hidlib_t *hl, live_script_t *lvs)
 	return lctx.vl.used;
 }
 #else
-static int lvs_list_langs(rnd_hidlib_t *hl, live_script_t *lvs)
+static int lvs_list_langs(rnd_design_t *hl, live_script_t *lvs)
 {
 	vtp0_t vl, ve;
 
@@ -213,7 +213,7 @@ static void lvs_button_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t
 	rnd_actionva(lvs->hidlib, "livescript", arg, lvs->name, NULL);
 }
 
-static live_script_t *rnd_dlg_live_script(rnd_hidlib_t *hidlib, const char *name)
+static live_script_t *rnd_dlg_live_script(rnd_design_t *hidlib, const char *name)
 {
 	rnd_hid_dad_buttons_t clbtn[] = {{"Close", 0}, {NULL, 0}};
 	char *title;
@@ -301,7 +301,7 @@ static int live_stop(live_script_t *lvs)
 
 #include "glue_undo.c"
 
-static int live_run(rnd_hidlib_t *hl, live_script_t *lvs)
+static int live_run(rnd_design_t *hl, live_script_t *lvs)
 {
 	rnd_hid_attribute_t *atxt = &lvs->dlg[lvs->wtxt];
 	rnd_hid_text_t *txt = atxt->wdata;
@@ -361,7 +361,7 @@ static const char *live_default_ext(live_script_t *lvs)
 	return NULL;
 }
 
-static int live_undo(rnd_hidlib_t *hl, live_script_t *lvs)
+static int live_undo(rnd_design_t *hl, live_script_t *lvs)
 {
 	if (lvs->undo_pre == lvs->undo_post)
 		return 0; /* the script did nothing */
@@ -375,7 +375,7 @@ static int live_undo(rnd_hidlib_t *hl, live_script_t *lvs)
 }
 
 
-static int live_load(rnd_hidlib_t *hl, live_script_t *lvs, const char *fn)
+static int live_load(rnd_design_t *hl, live_script_t *lvs, const char *fn)
 {
 	rnd_hid_attribute_t *atxt = &lvs->dlg[lvs->wtxt];
 	rnd_hid_text_t *txt = atxt->wdata;
@@ -417,7 +417,7 @@ static int live_load(rnd_hidlib_t *hl, live_script_t *lvs, const char *fn)
 	return 0;
 }
 
-static int live_save(rnd_hidlib_t *hl, live_script_t *lvs, const char *fn)
+static int live_save(rnd_design_t *hl, live_script_t *lvs, const char *fn)
 {
 	rnd_hid_attribute_t *atxt = &lvs->dlg[lvs->wtxt];
 	rnd_hid_text_t *txt = atxt->wdata;
@@ -479,7 +479,7 @@ fgw_error_t rnd_act_LiveScript(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 			RND_ACT_IRES(1);
 			return 0;
 		}
-		lvs = rnd_dlg_live_script(RND_ACT_HIDLIB, name);
+		lvs = rnd_dlg_live_script(RND_ACT_DESIGN, name);
 		if (lvs != NULL) {
 			htsp_set(&rnd_live_scripts, lvs->name, lvs);
 			RND_ACT_IRES(0);
@@ -493,7 +493,7 @@ fgw_error_t rnd_act_LiveScript(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		int n, len;
 
 		memset(&l, 0, sizeof(l));
-		len = lvs_list_langs(RND_ACT_HIDLIB, &l);
+		len = lvs_list_langs(RND_ACT_DESIGN, &l);
 		for(n = 0; n < len; n++) {
 			printf("'%s' '%s'\n", l.langs[n], l.lang_engines[n]);
 			free(l.langs[n]);
@@ -525,24 +525,24 @@ fgw_error_t rnd_act_LiveScript(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		RND_ACT_IRES(live_save(NULL, lvs, arg));
 	}
 	else if (rnd_strcasecmp(cmd, "undo") == 0) {
-		RND_ACT_IRES(live_undo(RND_ACT_HIDLIB, lvs));
+		RND_ACT_IRES(live_undo(RND_ACT_DESIGN, lvs));
 	}
 	else if (rnd_strcasecmp(cmd, "run") == 0) {
-		live_run(RND_ACT_HIDLIB, lvs);
+		live_run(RND_ACT_DESIGN, lvs);
 	}
 	else if (rnd_strcasecmp(cmd, "stop") == 0) {
 		live_stop(lvs);
 	}
 	else if (rnd_strcasecmp(cmd, "rerun") == 0) {
 		live_stop(lvs);
-		live_undo(RND_ACT_HIDLIB, lvs);
-		live_run(RND_ACT_HIDLIB, lvs);
+		live_undo(RND_ACT_DESIGN, lvs);
+		live_run(RND_ACT_DESIGN, lvs);
 	}
 	if (rnd_strcasecmp(cmd, "reload-rerun") == 0) {
 		RND_ACT_IRES(live_load(NULL, lvs, lvs->fn));
 		live_stop(lvs);
-		live_undo(RND_ACT_HIDLIB,lvs);
-		live_run(RND_ACT_HIDLIB, lvs);
+		live_undo(RND_ACT_DESIGN,lvs);
+		live_run(RND_ACT_DESIGN, lvs);
 	}
 
 	return 0;
