@@ -22,8 +22,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -32,9 +32,9 @@
 #include <stdio.h>
 
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hid_init.h>
+#include <librnd/hid/hid_init.h>
 
-static int perma_load(rnd_hidlib_t *hl, const char *dir, const char *id, const char *path_in, const char *lang)
+static int perma_load(rnd_design_t *hl, const char *dir, const char *id, const char *path_in, const char *lang)
 {
 	char spath[RND_PATH_MAX];
 	const char *path;
@@ -49,7 +49,7 @@ static int perma_load(rnd_hidlib_t *hl, const char *dir, const char *id, const c
 	return rnd_script_load(hl, id, path, lang);
 }
 
-static void perma_script_load_conf(rnd_hidlib_t *hl, const char *dir)
+static void perma_script_load_conf(rnd_design_t *hl, const char *dir)
 {
 	char path[RND_PATH_MAX], *errmsg;
 	lht_doc_t *doc;
@@ -70,7 +70,7 @@ static void perma_script_load_conf(rnd_hidlib_t *hl, const char *dir)
 	}
 
 	n = doc->root;
-	if ((n->type != LHT_LIST) || (strcmp(n->name, "pcb-rnd-perma-script-v1") != 0) || (strcmp(n->name, "librnd-perma-script-v1") != 0)) {
+	if ((n->type != LHT_LIST) || ((strcmp(n->name, "pcb-rnd-perma-script-v1") != 0) && (strcmp(n->name, "librnd-perma-script-v1") != 0))) {
 		rnd_message(RND_MSG_ERROR, "Failed to load script config '%s':\nroot node is not li:librnd-perma-script-v1\n", path);
 		goto end;
 	}
@@ -117,7 +117,7 @@ static void perma_script_load_conf(rnd_hidlib_t *hl, const char *dir)
 	lht_dom_uninit(doc);
 }
 
-static void perma_script_init(rnd_hidlib_t *hl)
+static void perma_script_init(rnd_design_t *hl)
 {
 	static int inited = 0;
 
@@ -129,7 +129,7 @@ static void perma_script_init(rnd_hidlib_t *hl)
 	inited = 1;
 }
 
-static void script_mainloop_perma_ev(rnd_hidlib_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
+static void script_mainloop_perma_ev(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
 {
 	if (rnd_hid_in_main_loop)
 		perma_script_init(hidlib);
