@@ -22,14 +22,14 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include <ctype.h>
-#include <librnd/core/hid_dad.h>
-#include <librnd/core/hid_dad_tree.h>
+#include <librnd/hid/hid_dad.h>
+#include <librnd/hid/hid_dad_tree.h>
 #include "live_script.h"
 
 /*** dialog box ***/
@@ -143,7 +143,7 @@ static void btn_unload_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t
 static void btn_reload_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
 {
 	script_dlg_t *ctx = caller_data;
-	rnd_hidlib_t *hl = rnd_gui->get_dad_hidlib(hid_ctx);
+	rnd_design_t *hl = rnd_gui->get_dad_design(hid_ctx);
 	rnd_hid_row_t *row = rnd_dad_tree_get_selected(&ctx->dlg[ctx->wslist]);
 	if (row == NULL)
 		return;
@@ -161,7 +161,7 @@ static void slist_cb(rnd_hid_attribute_t *attr, void *hid_ctx, rnd_hid_row_t *ro
 static void btn_load_cb(void *hid_ctx, void *caller_data, rnd_hid_attribute_t *attr)
 {
 	script_dlg_t *ctx = caller_data;
-	rnd_hidlib_t *hl = rnd_gui->get_dad_hidlib(hid_ctx);
+	rnd_design_t *hl = rnd_gui->get_dad_design(hid_ctx);
 	int failed;
 	char *tmp, *fn = rnd_hid_fileselect(rnd_gui, "script to load", "Select a script file to load", NULL, NULL, NULL, "script", RND_HID_FSD_READ, NULL);
 	rnd_hid_dad_buttons_t clbtn[] = {{"Cancel", -1}, {"ok", 0}, {NULL, 0}};
@@ -216,9 +216,8 @@ static void script_dlg_open(void)
 
 	RND_DAD_BEGIN_VBOX(script_dlg.dlg);
 	RND_DAD_COMPFLAG(script_dlg.dlg, RND_HATF_EXPFILL);
-	RND_DAD_BEGIN_HPANE(script_dlg.dlg);
+	RND_DAD_BEGIN_HPANE(script_dlg.dlg, "left-right");
 		RND_DAD_COMPFLAG(script_dlg.dlg, RND_HATF_EXPFILL);
-		RND_DAD_SET_PANE_NAME(script_dlg.dlg, "left-right");
 		/* left side */
 		RND_DAD_BEGIN_VBOX(script_dlg.dlg);
 			RND_DAD_COMPFLAG(script_dlg.dlg, RND_HATF_EXPFILL);
@@ -289,7 +288,7 @@ static fgw_error_t rnd_act_LoadScript(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 
 	ID_VALIDATE(id, LoadScript);
 
-	RND_ACT_IRES(rnd_script_load(RND_ACT_HIDLIB, id, fn, lang));
+	RND_ACT_IRES(rnd_script_load(RND_ACT_DESIGN, id, fn, lang));
 	script_dlg_update();
 	return 0;
 }
@@ -318,7 +317,7 @@ static fgw_error_t rnd_act_ReloadScript(fgw_arg_t *res, int argc, fgw_arg_t *arg
 
 	ID_VALIDATE(id, ReloadScript);
 
-	RND_ACT_IRES(script_reload(RND_ACT_HIDLIB, id));
+	RND_ACT_IRES(script_reload(RND_ACT_DESIGN, id));
 	script_dlg_update();
 	return 0;
 }
@@ -420,7 +419,7 @@ static fgw_error_t rnd_act_Oneliner(fgw_arg_t *res, int argc, fgw_arg_t *argv)
 		return 0;
 	}
 
-	RND_ACT_IRES(script_oneliner(RND_ACT_HIDLIB, lang, scr));
+	RND_ACT_IRES(script_oneliner(RND_ACT_DESIGN, lang, scr));
 	return 0;
 }
 
@@ -430,7 +429,7 @@ static fgw_error_t rnd_act_ActionString(fgw_arg_t *res, int argc, fgw_arg_t *arg
 {
 	const char *act;
 	RND_ACT_CONVARG(1, FGW_STR, ActionString, act = argv[1].val.str);
-	return rnd_parse_command_res(RND_ACT_HIDLIB, res, act, 1);
+	return rnd_parse_command_res(RND_ACT_DESIGN, res, act, 1);
 }
 
 static const char rnd_acth_rnd_math0[] = "No-argument math functions";
