@@ -107,23 +107,6 @@ RND_INLINE int pa_big_coord_near_zero(pa_big_coord_t c)
 	return 1;
 }
 
-/* Returns whether a==b within +-1 coord units to overcome rounding errors */
-RND_INLINE int pa_big_vect_equ_round(pa_big_vector_t a, pa_big_vector_t b)
-{
-	pa_big_coord_t dx, dy;
-
-	big_subn(dx, a.x, b.x, W, 0);
-	if (!pa_big_coord_near_zero(dx))
-		return 0;
-
-	big_subn(dy, a.y, b.y, W, 0);
-	if (!pa_big_coord_near_zero(dy))
-		return 0;
-
-	return 1;
-}
-
-
 /* Compares pt's point to vn and returns 1 if they match (0 otherwise). If
    vn has a cvc, use its high precision coords */
 RND_INLINE int pa_big_vnode_vect_equ(rnd_vnode_t *vn, pa_big_vector_t pt)
@@ -140,27 +123,6 @@ RND_INLINE int pa_big_vnode_vect_equ(rnd_vnode_t *vn, pa_big_vector_t pt)
 	return (pa_big_to_coord(pt.x) == vn->point[0]) && (pa_big_to_coord(pt.y) == vn->point[1]);
 }
 
-/* Same as pa_big_vnode_vect_equ but allows very close nodes (+-1 bit on
-   x and y) to match to overcome the effect of rounding errors from div
-   and isc calculation */
-RND_INLINE int pa_big_vnode_vect_equ_round(rnd_vnode_t *vn, pa_big_vector_t pt)
-{
-	/* high prec if available */
-	if (vn->cvclst_prev != NULL)
-		return pa_big_vect_equ_round(pt, vn->cvclst_prev->isc);
-
-	/* if pt is high precision but vn point is not, and pt is not an integer
-	   coord then they must differ (in fractions) - but they may be close enough
-	   for a rounded-match */
-	if (!pa_big_vect_is_int(pt)) {
-		pa_big_vector_t vnp;
-		rnd_pa_big_load_cvc(&vnp, vn);
-		return pa_big_vect_equ_round(pt, vnp);
-	}
-
-	return (pa_big_to_coord(pt.x) == vn->point[0]) && (pa_big_to_coord(pt.y) == vn->point[1]);
-}
-
 #define Vequ2(a,b)       (memcmp((a),   (b),   sizeof(rnd_vector_t)) == 0)
 int pa_big_vnode_vnode_equ(rnd_vnode_t *vna, rnd_vnode_t *vnb)
 {
@@ -227,3 +189,25 @@ int pa_small_big_xy_eq(rnd_coord_t smallx, rnd_coord_t smally, pa_big_coord_t bi
 	return (smallx == bigx[W/2]) && (smally == bigy[W/2]);
 }
 
+
+int pa_big_vmandist_small(pa_big_vector_t *a, rnd_vnode_t *vnode, int copy)
+{
+	pa_big_coord_t dx, dy;
+	pa_big_vector_t b;
+
+	rnd_pa_big_load_cvc(&b, vnode);
+
+	big_subn(dx, a->x, b.x, W, 0);
+	if (!pa_big_coord_near_zero(dx))
+		return 0;
+
+	big_subn(dy, a->y, b.y, W, 0);
+	if (!pa_big_coord_near_zero(dy))
+		return 0;
+
+
+	if (copy)
+		*a = b;
+
+	return 1;
+}
