@@ -77,6 +77,7 @@ struct pa_conn_desc_s {
 	char side;                           /* 'P' for previous 'N' for next (the other endpoint of edge of interest from ->parent) */
 	pa_big_vector_t isc;                 /* (precise) coords of the intersection */
 	unsigned prelim:1;                   /* preliminary allocation: isc is set but other fields are blank */
+	unsigned ignore:1;                   /* do not consider this when jumping to the next/prev cvc - e.g. this is a stub */
 
 	rnd_vnode_t *PP_OTHER;               /* used only in big_postproc to link point-point overlap nodes */
 };
@@ -103,7 +104,12 @@ typedef enum pa_plinept_label_e {
 /* ugly hack: long-jump out to the error handler e (assumed to exist in
    current scope, normally received as a function argument) with a
    non-zero error code */
-#define pa_error(code)  longjmp(*(e), code)
+#define pa_error(code) \
+	do { \
+		if (rnd_polybool_assert_on_error_code) \
+			assert(!"error code: " #code); \
+		longjmp(*(e), code); \
+	} while(0)
 
 /* standard error codes for pa_error() */
 enum {
@@ -147,3 +153,6 @@ pa_conn_desc_t *pa_add_conn_desc_at(rnd_vnode_t *node, char poly, pa_conn_desc_t
 pa_conn_desc_t *pa_prealloc_conn_desc(pa_big_vector_t isc);
 
 int rnd_polyarea_boolean_free_nochk(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_polyarea_t **res, int op);
+
+int pa_pline_is_point_on_seg(pa_seg_t *s, rnd_vector_t pt);
+
