@@ -35,15 +35,16 @@
  *
  */
 
+int rnd_polybool_disable_autocheck = 0;
 
 /* If vertices have high precision coords, the actual output will differ
    slightly after rounding the coords. In other words rounding moves vertices.
    In some cases this may change the topology and cause self-intersecting
    polygons or zero-length edges. Remove them here. */
-RND_INLINE void pa_bool_postproc(rnd_polyarea_t **pa, int from_selfisc)
+RND_INLINE void pa_bool_postproc(rnd_polyarea_t **pa, int from_selfisc, int papa_touch_risk)
 {
 #ifdef PA_BIGCOORD_ISC
-	pa_big_bool_postproc(pa, from_selfisc);
+	pa_big_bool_postproc(pa, from_selfisc, papa_touch_risk);
 #endif
 	/* else don't do naything: this happens only with big coords */
 }
@@ -70,6 +71,7 @@ int rnd_polyarea_boolean_free_nochk(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_
 	rnd_pline_t *a_isected = NULL, *holes = NULL;
 	jmp_buf e;
 	int code, from_selfisc = a_->from_selfisc && b_->from_selfisc;
+	int papa_touch_risk = 0;
 
 	*res = NULL;
 
@@ -110,7 +112,7 @@ int rnd_polyarea_boolean_free_nochk(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_
 #endif
 
 		/* intersect needs to make a list of the contours in a and b which are intersected */
-		pa_polyarea_intersect(&e, a, b, rnd_true);
+		pa_polyarea_intersect(&e, a, b, op, rnd_true, &papa_touch_risk);
 
 		/* We could speed things up a lot here if we only processed the relevant contours */
 		/* (Relevant parts of a are labeled below) */
@@ -120,13 +122,13 @@ int rnd_polyarea_boolean_free_nochk(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_
 		pa_polyarea_update_primary(&e, res, &holes, op, b);
 		pa_polyarea_separate_intersected(&e, res, &holes, &a_isected);
 		pa_polyarea_label_pline(a_isected, b, rnd_false);
-		pa_polyarea_collect_separated(&e, a_isected, res, &holes, op, rnd_false);
+		pa_polyarea_collect_separated(&e, a_isected, res, &holes, op, rnd_false, b);
 		pa_collect_b_area(&e, b, res, &holes, op);
 
 		pa_polyarea_free_all(&b);
 		rnd_poly_plines_free(&a_isected);
 
-		rnd_poly_insert_holes(&e, *res, &holes);
+		rnd_poly_insert_holes(&e, *res, &holes, op);
 	}
 
 	rnd_poly_plines_free(&holes); /* delete holes if any left (they are already inserted) */
@@ -137,7 +139,7 @@ int rnd_polyarea_boolean_free_nochk(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_
 	}
 
 	if (*res != NULL)
-		pa_bool_postproc(res, from_selfisc);
+		pa_bool_postproc(res, from_selfisc, papa_touch_risk);
 
 	return code;
 }
@@ -146,7 +148,7 @@ int rnd_polyarea_boolean_free(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_polyar
 {
 	int code = rnd_polyarea_boolean_free_nochk(a_, b_, res, op);
 
-	if ((code == 0) && (*res != NULL)) {
+	if ((code == 0) && (*res != NULL) && !rnd_polybool_disable_autocheck) {
 		assert(rnd_poly_valid(*res));
 	}
 
@@ -185,14 +187,24 @@ int rnd_polyarea_and_subtract_free(rnd_polyarea_t *a, rnd_polyarea_t *b, rnd_pol
 
 	code = setjmp(e);
 	if (code == 0) {
+		int papa_touch_risk = 0;
+
 		/* map and label */
-		pa_polyarea_intersect(&e, a, b, rnd_true);
+		pa_polyarea_intersect(&e, a, b, RND_PBO_ISECT, rnd_true, &papa_touch_risk);
 		pa_polyarea_label(a, b, rnd_false);
 		pa_polyarea_label(b, a, rnd_false);
 
 		/* calculate aandb */
 		pa_polyarea_collect(&e, a, aandb, &holes, RND_PBO_ISECT, rnd_false);
-		rnd_poly_insert_holes(&e, *aandb, &holes);
+		rnd_poly_insert_holes(&e, *aandb, &holes, RND_PBO_ISECT);
+
+#if 0
+		/* breaks test case work/test_poly/dicer1.lht; this function is used only
+		   in the dicer with vertical cutting edges, it is safe to skip this */
+		if (*aandb != NULL)
+			pa_bool_postproc(*aandb, 0, papa_touch_risk);
+#endif
+
 		assert(rnd_poly_valid(*aandb));
 
 		/* clean up temporary marks and hole list */
@@ -202,11 +214,21 @@ int rnd_polyarea_and_subtract_free(rnd_polyarea_t *a, rnd_polyarea_t *b, rnd_pol
 
 		/* calculate aminusb */
 		pa_polyarea_collect(&e, a, aminusb, &holes, RND_PBO_SUB, rnd_false);
-		rnd_poly_insert_holes(&e, *aminusb, &holes);
+		rnd_poly_insert_holes(&e, *aminusb, &holes, RND_PBO_SUB);
+
+#if 0
+		/* breaks test case work/test_poly/dicer1.lht; this function is used only
+		   in the dicer with vertical cutting edges, it is safe to skip this */
+		if (*aminusb != NULL)
+			pa_bool_postproc(*aminusb, 0, papa_touch_risk);
+#endif
+
 		assert(rnd_poly_valid(*aminusb));
 
 		pa_polyarea_free_all(&a);
 		pa_polyarea_free_all(&b);
+
+		TODO("postproc with papa_touch_risk");
 	}
 
 	rnd_poly_plines_free(&holes); /* delete holes if any left (not inserted) */
