@@ -85,7 +85,7 @@ rnd_bool rnd_polyarea_touching(rnd_polyarea_t *a, rnd_polyarea_t *b)
 #ifdef DEBUG
 		if (!rnd_poly_valid(a) || !rnd_poly_valid(b)) return -1;
 #endif
-		pa_polyarea_intersect(&e, a, b, rnd_false);
+		pa_polyarea_intersect(&e, a, b, -1, rnd_false, NULL);
 
 		if (pa_polyarea_label(a, b, rnd_true)) return rnd_true;
 		if (pa_polyarea_label(b, a, rnd_true)) return rnd_true;
@@ -102,7 +102,7 @@ typedef enum pa_island_isc_e {
 } pa_island_isc_t;
 
 
-/* Iterates through the nodes of small figuring if small is fully within big. */
+/* Iterates through the nodes of plines figuring if small is fully within big. */
 static pa_island_isc_t pa_pline_pline_isc(const rnd_pline_t *big, const rnd_pline_t *small, int point_on_edge_is_in)
 {
 	int got_in = 0, got_out = 0;
@@ -125,6 +125,14 @@ static pa_island_isc_t pa_pline_pline_isc(const rnd_pline_t *big, const rnd_plin
 			return PA_ISLAND_ISC_CROSS;
 	} while((n = n->next) != small->head);
 
+	/* Corner case: the big one can have a node within the small without the
+	   small having a node within the big. Test case: chk_hole_cntr */
+	n = big->head;
+	do {
+		if (pa_pline_is_vnode_inside(small, n, 0))
+			return PA_ISLAND_ISC_CROSS;
+	} while((n = n->next) != big->head);
+
 	/* if we got here, small is either full inside or fully outside */
 	return got_in ? PA_ISLAND_ISC_IN : PA_ISLAND_ISC_OUT;
 }
@@ -198,6 +206,22 @@ rnd_bool rnd_polyarea_island_isc(const rnd_polyarea_t *a, const rnd_polyarea_t *
 		}
 	} while((n = n->next) != pla->head);
 
+	/* check each outline vertex of plb whether it is inside of pla; if any is
+	   inside, plb is either inside pla or intersects pla */
+	n = plb->head;
+	do {
+		if (pa_pline_is_vnode_inside(pla, n, 0)) {
+			int r = pa_polyarea_pline_island_isc(b, plb);
+			switch(r) {
+				case PA_ISLAND_ISC_IN:    return 0; /* plb island is fully in an island of pla -> no intersection possible */
+				case PA_ISLAND_ISC_OUT:   return 1; /* plb island is fully outside of any island of pla -> it is in the solid area */
+				case PA_ISLAND_ISC_CROSS: return 1; /* crossing an island boundary means some points are inside the solid */
+			}
+			assert("!invalid return by pa_polyarea_is_vnode_island_isc()");
+			return 1;
+		}
+	} while((n = n->next) != plb->head);
+
 	return 0; /* no intersections */
 }
 
