@@ -183,15 +183,19 @@ RND_INLINE void pa_inshole_build_rtree(rnd_polyarea_t *src, pa_insert_holes_t *a
 }
 
 
+static rnd_pline_t orp_cont;
+
 /* Search for the container of pl. Also loads insh_ctx as a side effect */
 RND_INLINE rnd_pline_t *pa_inshole_find_container(jmp_buf *e, rnd_polyarea_t *dst, rnd_rtree_t *tree, rnd_pline_t *pl, pa_insert_holes_t **insh_ctx, int *risky)
 {
 	rnd_heap_t *heap;
 	rnd_pline_t *container = NULL;
+	int orp;
 
 	/* build a heap of all of the polys that the hole is inside its bounding box */
 	heap = rnd_heap_create();
 	rnd_r_search(tree, (rnd_box_t *)pl, NULL, pa_inshole_heap_it_cb, heap, NULL);
+	orp = pl->flg.orphaned;
 	if (rnd_heap_is_empty(heap)) {
 #ifndef NDEBUG
 #ifdef DEBUG
@@ -199,16 +203,31 @@ RND_INLINE rnd_pline_t *pa_inshole_find_container(jmp_buf *e, rnd_polyarea_t *ds
 #endif
 #endif
 		pa_pline_free(&pl);
+		rnd_heap_destroy(&heap);
+
+		/* do not panic if a hole of an removed-island (oprhaned hole) didn't
+		   find a container. Test case: gixedy */
+		if (orp)
+			return &orp_cont;
+
 		pa_error(pa_err_bad_parm);
 	}
 
+	pl->flg.orphaned = 0;
+
 	/* Search the heap for the container. */
 	*insh_ctx = (pa_insert_holes_t *)rnd_heap_remove_smallest(heap);
 
 	if (rnd_heap_is_empty(heap)) {
 		/* only one possibility it must be the right one */
-		if (!pa_pline_inside_pline((*insh_ctx)->pa->contours, pl))
+		if (!pa_pline_inside_pline((*insh_ctx)->pa->contours, pl)) {
+			if (orp) {
+				/* orphaned hole did not fit in any of the remaining contours, just
+				   drop it. Test case: gixed3 */
+				return &orp_cont;
+			}
 			*risky = 1;
+		}
 		container = (*insh_ctx)->pa->contours;
 	}
 	else {
@@ -223,12 +242,19 @@ RND_INLINE rnd_pline_t *pa_inshole_find_container(jmp_buf *e, rnd_polyarea_t *ds
 		}
 	}
 	rnd_heap_destroy(&heap);
+
+	/* if we had a non-empty heap and didn't find anything and our pl is orphaned
+	   that's all fine, that only means the parent fully got removed. Test case:
+	   gixedy2 */
+	if (orp && container == NULL)
+		return &orp_cont;
+
 	return container;
 }
 
 /* Src is a list of holes; try inserting them in dst; in *src return holes that
    are not inserted  */
-void rnd_poly_insert_holes(jmp_buf *e, rnd_polyarea_t *dst, rnd_pline_t **src)
+void rnd_poly_insert_holes(jmp_buf *e, rnd_polyarea_t *dst, rnd_pline_t **src, int op)
 {
 	rnd_pline_t *pl, *container;
 	rnd_rtree_t *tree;
@@ -236,7 +262,13 @@ void rnd_poly_insert_holes(jmp_buf *e, rnd_polyarea_t *dst, rnd_pline_t **src)
 	pa_insert_holes_t *all_insh_ctx, *insh_ctx;
 
 	if (*src == NULL) return; /* empty hole list */
-	if (dst == NULL) pa_error(pa_err_bad_parm); /* empty contour list */
+	if (dst == NULL) {
+		for(pl = *src; pl != NULL; pl = pl->next) {
+			if (!pl->flg.orphaned)
+				pa_error(pa_err_bad_parm); /* empty output contour list, nothing to insert into */
+		}
+		return; /* all holes are orphaned, no need to insert them anyway; test case: gixed2b */
+	}
 
 	num_polyareas = pa_polyarea_count(dst);
 
@@ -252,7 +284,14 @@ void rnd_poly_insert_holes(jmp_buf *e, rnd_polyarea_t *dst, rnd_pline_t **src)
 		*src = pl->next;
 
 		container = pa_inshole_find_container(e, dst, tree, pl, &insh_ctx, &risky);
+		if (container == &orp_cont)
+			continue;
+
 		if (container == NULL) {
+			if (op == RND_PBO_SUB) {
+				/* we probably have removed the section where this hole was in; test case: gixed3b */
+				continue;
+			}
 #ifndef NDEBUG
 #ifdef DEBUG
 			pa_poly_dump(dst);
