@@ -60,7 +60,7 @@ RND_INLINE void DEBUGP(const char *fmt, ...) { }
 #	define DEBUG_ANGLE_EN 0
 #endif
 
-#if defined(DEBUG) || DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP
+#if defined(DEBUG) || DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP || DEBUG_PA_DUMP_PA
 
 static const char *node_label_to_str(rnd_vnode_t *v)
 {
@@ -125,7 +125,7 @@ RND_INLINE void pa_debug_print_pline_labels(rnd_pline_t *a)
 RND_INLINE void pa_debug_print_pline_labels(rnd_pline_t *a) {}
 #endif
 
-#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP
+#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP || DEBUG_PA_DUMP_PA
 RND_INLINE void pa_debug_print_vnode_coord(rnd_vnode_t *n)
 {
 #ifdef PA_BIGCOORD_ISC
@@ -138,7 +138,7 @@ RND_INLINE void pa_debug_print_vnode_coord(rnd_vnode_t *n)
 }
 #endif
 
-#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP || DEBUG_ANGLE_EN
+#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP || DEBUG_ANGLE_EN || DEBUG_PA_DUMP_PA
 void pa_debug_print_angle(pa_big_angle_t a)
 {
 #ifdef PA_BIGCOORD_ISC
@@ -149,19 +149,19 @@ DEBUGP("%.09f", a);
 }
 #endif
 
-#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP
-RND_INLINE void pa_debug_print_cvc(pa_conn_desc_t *head)
+#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP || DEBUG_PA_DUMP_PA
+RND_INLINE void pa_debug_print_cvc_(pa_conn_desc_t *head, const char *ind)
 {
 	pa_conn_desc_t *n = head;
 
-	DEBUGP("CVC:\n");
+	DEBUGP("%sCVC:\n", ind);
 	if (n == NULL) {
-		DEBUGP(" (empty cvc)\n");
+		DEBUGP("%s (empty cvc)\n", ind);
 		return;
 	}
 
 	do {
-		DEBUGP(" %c %c ", n->poly, n->side);
+		DEBUGP("%s %c %c ", ind, n->poly, n->side);
 		pa_debug_print_angle(n->angle);
 		pa_debug_print_vnode_coord(n->parent);
 		if (n->side == 'N') {
@@ -174,6 +174,14 @@ RND_INLINE void pa_debug_print_cvc(pa_conn_desc_t *head)
 		}
 	} while((n = n->next) != head);
 }
+#endif
+
+#if DEBUG_CVC || DEBUG_DUMP || DEBUG_PAISC_DUMP
+RND_INLINE void pa_debug_print_cvc(pa_conn_desc_t *head)
+{
+	pa_debug_print_cvc_(head, "");
+}
+
 #else
 RND_INLINE void pa_debug_print_cvc(pa_conn_desc_t *conn_list) {}
 #endif
@@ -182,9 +190,22 @@ RND_INLINE void pa_debug_print_cvc(pa_conn_desc_t *conn_list) {}
 RND_INLINE void pa_debug_print_isc(int num_isc, const char *name, pa_big_vector_t isc1, pa_big_vector_t isc2, rnd_vnode_t *a1, rnd_vnode_t *a2, rnd_vnode_t *b1, rnd_vnode_t *b2)
 {
 #ifdef PA_BIGCOORD_ISC
+	int i;
 	DEBUGP("ISC %s: %.03f;%.03f..%.03f;%.03f and %.03f;%.03f..%.03f;%.03f\n", name, pa_big_vnxd(a1), pa_big_vnyd(a1), pa_big_vnxd(a2), pa_big_vnyd(a2), pa_big_vnxd(b1), pa_big_vnyd(b1), pa_big_vnxd(b2), pa_big_vnyd(b2));
-	if (num_isc > 0) DEBUGP(" %.03f;%.03f\n", pa_big_double(isc1.x), pa_big_double(isc1.y));
-	if (num_isc > 1) DEBUGP(" %.03f;%.03f\n", pa_big_double(isc2.x), pa_big_double(isc2.y));
+	if (num_isc > 0) {
+		DEBUGP(" %.03f;%.03f (x=", pa_big_double(isc1.x), pa_big_double(isc1.y));
+		for(i = 0; i < PA_BIGCRD_WIDTH; i++) DEBUGP("%ld ", (long)isc1.x[i]);
+		DEBUGP("y=");
+		for(i = 0; i < PA_BIGCRD_WIDTH; i++) DEBUGP("%ld ", (long)isc1.y[i]);
+		DEBUGP(")\n");
+	}
+	if (num_isc > 1) {
+		DEBUGP(" %.03f;%.03f (x=", pa_big_double(isc2.x), pa_big_double(isc2.y));
+		for(i = 0; i < PA_BIGCRD_WIDTH; i++) DEBUGP("%ld ", (long)isc2.x[i]);
+		DEBUGP("y=");
+		for(i = 0; i < PA_BIGCRD_WIDTH; i++) DEBUGP("%ld ", (long)isc2.y[i]);
+		DEBUGP(")\n");
+	}
 #else
 	DEBUGP("ISC %s: %$mD..%$mD and %$mD..%$mD\n", name, a1->point[0], a1->point[1], a2->point[0], a2->point[1], b1->point[0], b1->point[1], b2->point[0], b2->point[1]);
 	if (num_isc > 0) DEBUGP(" %$mD\n", isc1[0], isc1[1]);
@@ -211,28 +232,45 @@ typedef enum { /* bitfield of extra info the dump should contain */
 	PA_DBG_DUMP_CVC = 1
 } pa_debug_dump_extra_t;
 
-#if DEBUG_DUMP || DEBUG_PAISC_DUMP
+#if DEBUG_DUMP || DEBUG_PAISC_DUMP || DEBUG_PA_DUMP_PA
 
-RND_INLINE void pa_debug_dump_vnode_coord(FILE *f, rnd_vnode_t *n)
+RND_INLINE void pa_debug_dump_vnode_coord(FILE *f, rnd_vnode_t *n, pa_debug_dump_extra_t extra)
 {
 #ifdef PA_BIGCOORD_ISC
 	if (n->cvclst_next != NULL) {
-		fprintf(f, "   %.012f %.012f\n", pa_big_double(n->cvclst_next->isc.x), pa_big_double(n->cvclst_next->isc.y));
-		return;
+		int i;
+
+		fprintf(f, "   %.012f %.012f (x=", pa_big_double(n->cvclst_next->isc.x), pa_big_double(n->cvclst_next->isc.y));
+			for(i = 0; i < PA_BIGCRD_WIDTH; i++) fprintf(f, "%ld ", (long)n->cvclst_next->isc.x[i]);
+			fprintf(f, "y=");
+			for(i = 0; i < PA_BIGCRD_WIDTH; i++) fprintf(f, "%ld ", (long)n->cvclst_next->isc.y[i]);
+		fprintf(f, ")\n");
+		if (extra & PA_DBG_DUMP_CVC) {
+			pa_debug_print_cvc_(n->cvclst_next, "    ");
+		}
 	}
 #endif
 	fprintf(f, "   %ld %ld\n", (long)n->point[0], (long)n->point[1]);
 }
 
-RND_INLINE void pa_debug_dump_pline(FILE *f, rnd_pline_t *pl, pa_debug_dump_extra_t extra)
+RND_INLINE void pa_debug_dump_pline_from(FILE *f, rnd_vnode_t *v, pa_debug_dump_extra_t extra)
 {
-	rnd_vnode_t *v = pl->head;
+	rnd_vnode_t *start = v;
 	do {
-		pa_debug_dump_vnode_coord(f, v);
-	} while((v = v->next) != pl->head);
+		if (v == NULL) {
+			fprintf(f, "   <NULL>\n");
+			break;
+		}
+		pa_debug_dump_vnode_coord(f, v, extra);
+	} while((v = v->next) != start);
 }
 
-static void pa_debug_dump(FILE *f, const char *title, rnd_polyarea_t *pa, pa_debug_dump_extra_t extra)
+RND_INLINE void pa_debug_dump_pline(FILE *f, rnd_pline_t *pl, pa_debug_dump_extra_t extra)
+{
+	pa_debug_dump_pline_from(f, pl->head, extra);
+}
+
+static void pa_debug_dump_(FILE *f, const char *title, rnd_polyarea_t *pa, pa_debug_dump_extra_t extra)
 {
 	rnd_pline_t *pl;
 	rnd_polyarea_t *pn = pa;
@@ -264,9 +302,22 @@ static void pa_debug_dump(FILE *f, const char *title, rnd_polyarea_t *pa, pa_deb
 	fprintf(f, " End\n\n");
 }
 
+#undef fopen
+void pa_dump_pa(rnd_polyarea_t *pa, const char *fn)
+{
+	FILE *f = fopen(fn, "w");
+	pa_debug_dump_(f, NULL, pa, 0);
+	fclose(f);
+}
+#endif
+
+#if DEBUG_DUMP || DEBUG_PAISC_DUMP
+static void pa_debug_dump(FILE *f, const char *title, rnd_polyarea_t *pa, pa_debug_dump_extra_t extra)
+{
+	pa_debug_dump_(f, title, pa, extra);
+}
 #else
 static void pa_debug_dump(FILE *f, const char *title, rnd_polyarea_t *pa, pa_debug_dump_extra_t extra) {}
-
 #endif
 
 #if DEBUG_GATHER || DEBUG_JUMP
@@ -309,3 +360,30 @@ static void pa_debug_dump(FILE *f, const char *title, rnd_polyarea_t *pa, pa_deb
 #	undef DEBUG_GATHER
 #	define DEBUG_GATHER PA_DEBUGP_DUMMY
 #endif
+
+
+#if DEBUG_SELFISC
+#	undef DEBUG_SELFISC
+#	define DEBUG_SELFISC DEBUGP
+#else
+#	undef DEBUG_SELFISC
+#	define DEBUG_SELFISC PA_DEBUGP_DUMMY
+#endif
+
+#if DEBUG_CLIP
+#	undef DEBUG_CLIP
+#	define DEBUG_CLIP DEBUGP
+#else
+#	undef DEBUG_CLIP
+#	define DEBUG_CLIP PA_DEBUGP_DUMMY
+#endif
+
+#if DEBUG_SLICE
+#	undef DEBUG_SLICE
+#	define DEBUG_SLICE DEBUGP
+#	define WANT_DEBUG_SLICE
+#else
+#	undef DEBUG_SLICE
+#	define DEBUG_SLICE PA_DEBUGP_DUMMY
+#	undef WANT_DEBUG_SLICE
+#endif
