@@ -46,11 +46,12 @@ typedef enum pa_dic_pline_label_e { /* pline's flg.label */
 	PA_PLD_AWAY = 4        /* pline is fully outside but not wrapping the box */
 } pa_dic_pline_label_t;
 
+TODO("x and y should be big-coord instead of double")
 struct pa_dic_isc_s {
 	rnd_vnode_t *vn;
 	rnd_pline_t *pl;
 
-	rnd_coord_t x, y;      /* corners have vn==NULL pl==NULL but we still need to remember the coords */
+	double x, y;      /* corners have vn==NULL pl==NULL but we still need to remember the coords */
 
 	unsigned temporary:1;  /* inserted by the dicer code, shall be removed at the end */
 	unsigned coax:1;       /* set if line is coaxial (may overlap with) the edge */
@@ -59,7 +60,7 @@ struct pa_dic_isc_s {
 	pa_dic_isc_t *next;
 };
 
-TODO("cache allocations");
+TODO("cache allocations")
 RND_INLINE pa_dic_isc_t *pa_dic_isc_alloc(pa_dic_ctx_t *ctx)
 {
 	return malloc(sizeof(pa_dic_isc_t));
@@ -172,7 +173,7 @@ RND_INLINE rnd_vnode_t *pa_dic_split_seg(pa_dic_ctx_t *ctx, pa_seg_t *seg, rnd_c
 
 /* Record an intersection, potentially creating a new temporary node in seg;
    first and second are the first and second real (non-temporary) node of seg */
-RND_INLINE pa_dic_isc_t *pa_dic_isc(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rnd_coord_t x, rnd_coord_t y, int *iscs, int coax, rnd_vnode_t *first, rnd_vnode_t *second)
+RND_INLINE pa_dic_isc_t *pa_dic_isc(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, double x, double y, int *iscs, int coax, rnd_vnode_t *first, rnd_vnode_t *second)
 {
 	pa_dic_isc_t *isc = pa_dic_isc_alloc(ctx);
 	rnd_vnode_t *nd = NULL;
@@ -214,17 +215,17 @@ RND_INLINE pa_dic_isc_t *pa_dic_isc(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_sid
 	return isc;
 }
 
-TODO("rewrite these with big_coords to make the 64-bit-coord safe");
-rnd_coord_t pa_line_x_for_y(rnd_coord_t lx1, rnd_coord_t ly1, rnd_coord_t lx2, rnd_coord_t ly2, rnd_coord_t y)
+TODO("rewrite these with big_coords to make the 64-bit-coord safe (replace dobule)")
+double pa_line_x_for_y(rnd_coord_t lx1, rnd_coord_t ly1, rnd_coord_t lx2, rnd_coord_t ly2, rnd_coord_t y)
 {
 	double dx = (double)(lx2 - lx1) / (double)(ly2 - ly1);
-	return rnd_round(lx1 + (y - ly1) * dx);
+	return (double)lx1 + (double)(y - ly1) * dx;
 }
 
-rnd_coord_t pa_line_y_for_x(rnd_coord_t lx1, rnd_coord_t ly1, rnd_coord_t lx2, rnd_coord_t ly2, rnd_coord_t x)
+double pa_line_y_for_x(rnd_coord_t lx1, rnd_coord_t ly1, rnd_coord_t lx2, rnd_coord_t ly2, rnd_coord_t x)
 {
 	double dy = (double)(ly2 - ly1) / (double)(lx2 - lx1);
-	return rnd_round(ly1 + (x - lx1) * dy);
+	return (double)ly1 + (double)(x - lx1) * dy;
 }
 
 
@@ -269,7 +270,7 @@ static int pa_dic_isc_h(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rn
 	}
 	else {
 		/* normal case: sloped line */
-		rnd_coord_t x;
+		double x;
 
 		if (ly1 > ly2) {
 			rnd_swap(rnd_coord_t, ly1, ly2);
@@ -332,7 +333,7 @@ static int pa_dic_isc_v(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rn
 	}
 	else {
 		/* normal case: sloped line */
-		rnd_coord_t y;
+		double y;
 
 		if (lx1 > lx2) {
 			rnd_swap(rnd_coord_t, lx1, lx2);
@@ -778,8 +779,7 @@ RND_INLINE int pa_dic_wrong_turn(pa_dic_ctx_t *ctx, rnd_vnode_t *en, pa_dic_isc_
    intersection where the edge went outside */
 RND_INLINE pa_dic_isc_t *pa_dic_gather_pline(pa_dic_ctx_t *ctx, rnd_vnode_t *start, pa_dic_isc_t *start_isc, pa_dic_isc_t *term)
 {
-	pa_dic_pt_box_relation_t state = PA_DPT_ON_EDGE, dir;
-	rnd_vnode_t *prev = NULL;
+	pa_dic_pt_box_relation_t dir;
 	rnd_vnode_t *n;
 	pa_dic_isc_t *si, *last_si = NULL, *pending_si = NULL;
 
@@ -831,7 +831,6 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_pline(pa_dic_ctx_t *ctx, rnd_vnode_t *sta
 
 		pa_dic_append(ctx, n->point[0], n->point[1]);
 		DEBUG_CLIP("       append: %ld;%ld\n", (long)n->point[0], (long)n->point[1]);
-		prev = n;
 		n = n->next;
 	} while(n != start);
 
@@ -847,7 +846,7 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_edge(pa_dic_ctx_t *ctx, pa_dic_isc_t *sta
 	for(i = start_isc->next;; i = i->next) {
 		if (i == term) {
 			if (i == start_isc->next)
-				pa_dic_append(ctx, start_isc->x, start_isc->y); /* test case: clip26, 45;80..55;70..65;80 */
+				pa_dic_append(ctx, rnd_round(start_isc->x), rnd_round(start_isc->y)); /* test case: clip26, 45;80..55;70..65;80 */
 			DEBUG_CLIP("        break: term\n");
 			break;
 		}
@@ -855,8 +854,8 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_edge(pa_dic_ctx_t *ctx, pa_dic_isc_t *sta
 			DEBUG_CLIP("        break: already ecollected\n");
 			break;
 		}
-		pa_dic_append(ctx, i->x, i->y);
-		DEBUG_CLIP("       append: %ld;%ld\n", (long)i->x, (long)i->y);
+		pa_dic_append(ctx, rnd_round(i->x), rnd_round(i->y));
+		DEBUG_CLIP("       append: %ld;%ld\n", (long)rnd_round(i->x), (long)rnd_round(i->y));
 		if ((i->vn != NULL) && (pa_dic_emit_island_predict(ctx, i->vn, i->pl) == PA_DPT_INSIDE)) {
 			DEBUG_CLIP("        break: pline going inside\n");
 			break;
@@ -892,8 +891,8 @@ RND_INLINE void pa_dic_emit_island_collect_from(pa_dic_ctx_t *ctx, pa_dic_isc_t
 
 	/* it's safe to start here, next deviation is going inside */
 	pa_dic_begin(ctx);
-	pa_dic_append(ctx, from->x, from->y);
-	DEBUG_CLIP("       append: %ld;%ld\n", (long)from->x, (long)from->y);
+	pa_dic_append(ctx, rnd_round(from->x), rnd_round(from->y));
+	DEBUG_CLIP("       append: %ld;%ld\n", (long)rnd_round(from->x), (long)rnd_round(from->y));
 	from->pcollected = 1;
 
 	i = from;
@@ -976,6 +975,14 @@ RND_INLINE void pa_dic_emit_island(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 	rnd_pline_t *pl;
 
 	pl = pa->contours;
+
+	/* no output for a zero or negative sized box */
+	if ((ctx->clip.X1 >= ctx->clip.X2) || (ctx->clip.Y1 >= ctx->clip.Y2)) return;
+
+	/* if clipbox doesn't include pl's bbox at all, output is empty */
+	if ((ctx->clip.X1 >= pl->xmax) || (ctx->clip.Y1 >= pl->ymax)) return;
+	if ((ctx->clip.X2 <= pl->xmin) || (ctx->clip.Y2 <= pl->ymin)) return;
+
 	DEBUG_CLIP("  emit island: %ld;%ld\n", (long)pl->head->point[0], (long)pl->head->point[1]);
 	pa_dic_pline_label(ctx, pl);
 
@@ -1092,7 +1099,7 @@ RND_INLINE void pa_slc_map_pline_pa(pa_slc_ctx_t *ctx)
 
 static int cmp_slc_endp(const void *a_, const void *b_)
 {
-	pa_slc_endp_t *a = a_, *b = b_;
+	const pa_slc_endp_t *a = a_, *b = b_;
 	return (a->x < b->x) ? -1 : +1;
 }
 
@@ -1297,17 +1304,15 @@ static void pa_nhdic_append_coord(pa_dic_ctx_t *ctx, rnd_coord_t x, rnd_coord_t
 
 static void pa_nhdic_end_pline(pa_dic_ctx_t *ctx)
 {
-	rnd_polyarea_t *new_pa;
 	pa_nhdic_ctx_t *cctx = ctx->user_data;
 	assert(cctx->pl != NULL);
 
 	pa_pline_update(cctx->pl, 0);
-	assert(cctx->pl->flg.orient == RND_PLF_DIR);
 
-/* Alternatively, if it turns out it's legal to get inverted plines:
+
+	/* Always draw positive slices */
 	if (cctx->pl->flg.orient != RND_PLF_DIR)
 		pa_pline_invert(cctx->pl);
-*/
 
 	cctx->emit_pline(cctx->pl, cctx->user_data);
 
@@ -1319,7 +1324,7 @@ static void pa_nhdic_end_pline(pa_dic_ctx_t *ctx)
 
 void rnd_polyarea_no_holes_dicer(rnd_polyarea_t *pa, rnd_coord_t clipX1, rnd_coord_t clipY1, rnd_coord_t clipX2, rnd_coord_t clipY2, void (*emit)(rnd_pline_t *, void *), void *user_data)
 {
-	pa_dic_ctx_t ctx = {0};;
+	pa_dic_ctx_t ctx = {0};
 	pa_nhdic_ctx_t nh = {0};
 
 	nh.emit_pline = emit;
