@@ -70,6 +70,7 @@ typedef struct pa_seg_s {
 
 void rnd_poly_copy_edge_tree(rnd_pline_t *dst, const rnd_pline_t *src);
 
+#include "pb2_geo_raw.h"
 
 /* Returns whether pl intersects with the line segment specified in lx;ly; if
    it does, and cx and cy are not NULL, fill in cx;cy with the first
@@ -77,7 +78,7 @@ void rnd_poly_copy_edge_tree(rnd_pline_t *dst, const rnd_pline_t *src);
 rnd_bool rnd_pline_isect_line(rnd_pline_t *pl, rnd_coord_t lx1, rnd_coord_t ly1, rnd_coord_t lx2, rnd_coord_t ly2, rnd_coord_t *cx, rnd_coord_t *cy);
 
 /* Update the bbox of pl using pt's coords */
-RND_INLINE void pa_pline_box_bump(rnd_pline_t *pl, const rnd_vector_t pt)
+RND_INLINE void pa_pline_box_bump_pt(rnd_pline_t *pl, const rnd_vector_t pt)
 {
 	if (pt[0]     < pl->xmin) pl->xmin = pt[0];
 	if ((pt[0]+1) > pl->xmax) pl->xmax = pt[0]+1;
@@ -85,9 +86,26 @@ RND_INLINE void pa_pline_box_bump(rnd_pline_t *pl, const rnd_vector_t pt)
 	if ((pt[1]+1) > pl->ymax) pl->ymax = pt[1]+1;
 }
 
+RND_INLINE void pa_pline_box_bump_curve(rnd_pline_t *pl, rnd_vnode_t *v)
+{
+	rnd_rtree_box_t bb;
+
+	switch(v->flg.curve_type) {
+		case RND_VNODE_LINE: pa_pline_box_bump_pt(pl, v->point); break;
+		case RND_VNODE_ARC:
+			pb2_raw_arc_bbox(&bb, v->point, v->next->point, v->curve.arc.center, v->curve.arc.adir);
+			if (bb.x1 < pl->xmin) pl->xmin = bb.x1;
+			if (bb.y1 < pl->ymin) pl->ymin = bb.y1;
+			if (bb.x2 > pl->xmax) pl->xmax = bb.x2;
+			if (bb.y2 > pl->ymax) pl->ymax = bb.y2;
+			break;
+	}
+}
+
+
 int rnd_polyarea_boolean_free_nochk(rnd_polyarea_t *a_, rnd_polyarea_t *b_, rnd_polyarea_t **res, int op);
 
-int pa_pline_is_point_on_seg(pa_seg_t *s, rnd_vector_t pt);
+int pa_pline_is_point_on_curve(pa_seg_t *s, rnd_vector_t pt);
 
 void pa_calc_angle_nn(pa_angle_t *dst, rnd_vector_t PT, rnd_vector_t OTHER);
 
@@ -98,4 +116,6 @@ do { \
 	b = __tmp__; \
 } while(0)
 
+
+
 #endif
