@@ -34,7 +34,6 @@
 
 #include "pa_dicer.h"
 #include <librnd/core/vtc0.h>
-#include <librnd/core/error.h>
 
 /*** clipper ***/
 
@@ -132,7 +131,7 @@ RND_INLINE int crd_in_between(rnd_coord_t c, rnd_coord_t low, rnd_coord_t high)
 RND_INLINE int crd_in_between_auto(rnd_coord_t c, rnd_coord_t low, rnd_coord_t high)
 {
 	if (low > high)
-		rnd_swap(rnd_coord_t, low, high);
+		pa_swap(rnd_coord_t, low, high);
 	return (c >= low) && (c <= high);
 }
 
@@ -184,12 +183,13 @@ RND_INLINE pa_dic_isc_t *pa_dic_isc(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_sid
 
 
 	if (seg != NULL) {
-		if ((first->point[0] == x) && (first->point[1] == y))
+		rnd_coord_t cx = rnd_round(x), cy = rnd_round(y);
+		if ((first->point[0] == cx) && (first->point[1] == cy))
 			nd = first;
-		else if ((second->point[0] == x) && (second->point[1] == y))
+		else if ((second->point[0] == cx) && (second->point[1] == cy))
 			nd = second;
 		else
-			nd = pa_dic_split_seg(ctx, seg, x, y);
+			nd = pa_dic_split_seg(ctx, seg, cx, cy);
 		isc->vn = nd;
 		isc->pl = seg->p;
 		isc->temporary = nd->flg.TEMPORARY;
@@ -215,7 +215,7 @@ RND_INLINE pa_dic_isc_t *pa_dic_isc(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_sid
 	return isc;
 }
 
-TODO("rewrite these with big_coords to make the 64-bit-coord safe (replace dobule)")
+TODO("bignum: rewrite these with big_coords to make the 64-bit-coord safe (replace dobule)")
 double pa_line_x_for_y(rnd_coord_t lx1, rnd_coord_t ly1, rnd_coord_t lx2, rnd_coord_t ly2, rnd_coord_t y)
 {
 	double dx = (double)(lx2 - lx1) / (double)(ly2 - ly1);
@@ -251,7 +251,7 @@ static int pa_dic_isc_h(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rn
 		if (ly1 != y)
 			return 0;
 		if (lx1 > lx2)
-			rnd_swap(rnd_coord_t, lx1, lx2);
+			pa_swap(rnd_coord_t, lx1, lx2);
 		if ((lx1 > ctx->clip.X2) || (lx2 < ctx->clip.X1))
 			return 0;
 
@@ -273,8 +273,8 @@ static int pa_dic_isc_h(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rn
 		double x;
 
 		if (ly1 > ly2) {
-			rnd_swap(rnd_coord_t, ly1, ly2);
-			rnd_swap(rnd_coord_t, lx1, lx2);
+			pa_swap(rnd_coord_t, ly1, ly2);
+			pa_swap(rnd_coord_t, lx1, lx2);
 		}
 
 		if (y == ly1)
@@ -316,7 +316,7 @@ static int pa_dic_isc_v(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rn
 		if (lx1 != x)
 			return 0;
 		if (ly1 > ly2)
-			rnd_swap(rnd_coord_t, ly1, ly2);
+			pa_swap(rnd_coord_t, ly1, ly2);
 
 		if (crd_in_between(ly1, ctx->clip.Y1, ctx->clip.Y2))
 			pa_dic_isc(ctx, seg, side, x, ly1, &iscs, 1, first, second);
@@ -336,8 +336,8 @@ static int pa_dic_isc_v(pa_dic_ctx_t *ctx, pa_seg_t *seg, pa_dic_side_t side, rn
 		double y;
 
 		if (lx1 > lx2) {
-			rnd_swap(rnd_coord_t, lx1, lx2);
-			rnd_swap(rnd_coord_t, ly1, ly2);
+			pa_swap(rnd_coord_t, lx1, lx2);
+			pa_swap(rnd_coord_t, ly1, ly2);
 		}
 
 		if (x == lx1)
@@ -525,13 +525,13 @@ RND_INLINE void pa_dic_sort_sides(pa_dic_ctx_t *ctx)
 	ctx->head = ctx->corner[0];
 
 #if DEBUG_CLIP_DUMP_LOOP != 0
-	rnd_trace("isc loop #1:\n");
+	pa_trace("isc loop #1:\n", 0);
 	i = ctx->head;
 	do {
 		if (i->vn != NULL)
-			rnd_trace(" %ld;%ld .. %ld;%ld vn=%p pl=%p\n", i->x, i->y, i->vn->next->point[0], i->vn->next->point[1], i->vn, i->pl);
+			pa_trace(" ", Pdbl2(i->x, i->y), " .. ", Pvnodep(i->vn->next), " vn=", Pptr(i->vn), " pl=", Pptr(i->pl), "\n", 0);
 		else
-			rnd_trace(" %ld;%ld (E)\n", i->x, i->y);
+			pa_trace(" ", Pdbl2(i->x, i->y)," (E)\n", 0);
 	} while((i = i->next) != ctx->head);
 #endif
 
@@ -558,13 +558,13 @@ RND_INLINE void pa_dic_sort_sides(pa_dic_ctx_t *ctx)
 	} while((i = next) != ctx->head);
 
 #if DEBUG_CLIP_DUMP_LOOP != 0
-	rnd_trace("isc loop #2:\n");
+	pa_trace("isc loop #2:\n", 0);
 	i = ctx->head;
 	do {
 		if (i->vn != NULL)
-			rnd_trace(" %ld;%ld .. %ld;%ld vn=%p pl=%p\n", i->x, i->y, i->vn->next->point[0], i->vn->next->point[1], i->vn, i->pl);
+			pa_trace(" ", Pdbl2(i->x, i->y), " .. ", Pvnodep(i->vn->next), " vn=", Pptr(i->vn), " pl=", Pptr(i->pl), 0);
 		else
-			rnd_trace(" %ld;%ld (E)\n", i->x, i->y);
+			pa_trace(" ", Pdbl2(i->x, i->y), " (E)\n", 0);
 	} while((i = i->next) != ctx->head);
 #endif
 
@@ -672,7 +672,7 @@ RND_INLINE pa_dic_pt_box_relation_t pa_dic_emit_island_predict(pa_dic_ctx_t *ctx
 			rnd_coord_t midy = (n->point[1] + n->prev->point[1]) / 2;
 
 
-			TODO("Corner case: if difference is only 1 above, the midpoint is rounded badly - use bigcoord");
+			TODO("bignum: Corner case: if difference is only 1 above, the midpoint is rounded badly - use bigcoord");
 			/* special case: a pline seg sloping across two edge iscs, test case 
 			   clip01b; if the center of the line is inside the box (not on edge)
 			   we are good to go */
@@ -787,7 +787,7 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_pline(pa_dic_ctx_t *ctx, rnd_vnode_t *sta
 
 		dir = pa_dic_pt_in_box(n->point[0], n->point[1], &ctx->clip);
 		if (dir == PA_DPT_OUTSIDE) {
-			DEBUG_CLIP("        break: going outside\n");
+			DEBUG_CLIP("        break: going outside\n", 0);
 			return last_si;
 		}
 
@@ -800,7 +800,7 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_pline(pa_dic_ctx_t *ctx, rnd_vnode_t *sta
 			si = pa_dic_find_isc_for_node(ctx, n);
 
 			if (si == term) {
-				DEBUG_CLIP("        break: term\n");
+				DEBUG_CLIP("        break: term\n", 0);
 				return term;
 			}
 
@@ -811,14 +811,14 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_pline(pa_dic_ctx_t *ctx, rnd_vnode_t *sta
 					   when not returned to the starting point (test case: clip09) */
 					pa_dic_append(ctx, n->point[0], n->point[1]);
 				}
-				DEBUG_CLIP("        break: wrong turn\n");
+				DEBUG_CLIP("        break: wrong turn\n", 0);
 				return si;
 			}
 
 			/* break at hole/island overlapping edge; test case: clip21c/clip25c */
 			if (pa_dic_emit_island_predict(ctx, n, si->pl) == PA_DPT_OUTSIDE) {
 				pa_dic_append(ctx, n->point[0], n->point[1]);
-				DEBUG_CLIP("        break: overlapping edg\n");
+				DEBUG_CLIP("        break: overlapping edg\n", 0);
 				return si;
 			}
 
@@ -827,7 +827,7 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_pline(pa_dic_ctx_t *ctx, rnd_vnode_t *sta
 		}
 
 		pa_dic_append(ctx, n->point[0], n->point[1]);
-		DEBUG_CLIP("       append: %ld;%ld\n", (long)n->point[0], (long)n->point[1]);
+		DEBUG_CLIP("       append: ", Pvnodep(n), "\n", 0);
 		n = n->next;
 	} while(n != start);
 
@@ -844,17 +844,17 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_edge(pa_dic_ctx_t *ctx, pa_dic_isc_t *sta
 		if (i == term) {
 			if (i == start_isc->next)
 				pa_dic_append(ctx, rnd_round(start_isc->x), rnd_round(start_isc->y)); /* test case: clip26, 45;80..55;70..65;80 */
-			DEBUG_CLIP("        break: term\n");
+			DEBUG_CLIP("        break: term\n", 0);
 			break;
 		}
 		if (i->ecollected) {
-			DEBUG_CLIP("        break: already ecollected\n");
+			DEBUG_CLIP("        break: already ecollected\n", 0);
 			break;
 		}
 		pa_dic_append(ctx, rnd_round(i->x), rnd_round(i->y));
-		DEBUG_CLIP("       append: %ld;%ld\n", (long)rnd_round(i->x), (long)rnd_round(i->y));
+		DEBUG_CLIP("       append: ", Pcoord2(rnd_round(i->x), rnd_round(i->y)), "\n", 0);
 		if ((i->vn != NULL) && (pa_dic_emit_island_predict(ctx, i->vn, i->pl) == PA_DPT_INSIDE)) {
-			DEBUG_CLIP("        break: pline going inside\n");
+			DEBUG_CLIP("        break: pline going inside\n", 0);
 			break;
 		}
 		i->ecollected = 1;
@@ -862,6 +862,34 @@ RND_INLINE pa_dic_isc_t *pa_dic_gather_edge(pa_dic_ctx_t *ctx, pa_dic_isc_t *sta
 	return i;
 }
 
+/* returns 1 if ptx;pty is on the edge of the box */
+RND_INLINE int pa_dic_pt_on_box_edge(rnd_coord_t ptx, rnd_coord_t pty, rnd_box_t *box)
+{
+	return (ptx == box->X1) || (ptx == box->X2) || (pty == box->Y1) || (pty == box->Y2);
+}
+
+
+/* return 1 if vn is part of a hole */
+RND_INLINE int pa_dic_is_hole_predict(pa_dic_ctx_t *ctx, pa_dic_isc_t *i)
+{
+	rnd_vnode_t *vn;
+
+	if (i->pl->flg.orient != RND_PLF_INV)
+		return 0; /* accept: not a hole */
+
+	/* go around this hole; if we hit the box sooner than returning to the
+	   starting point, this will be part of the contour, otherwise it's
+	   a hole with one point toiching the contour (and in that case
+	   should be picked up starting from elsewhere). Note: by this point
+	   all intersections with the box have a temporary vnode inserted */
+	for(vn = i->vn->next; vn != i->vn; vn = vn->next)
+		if (pa_dic_pt_on_box_edge(vn->point[0], vn->point[1], &ctx->clip))
+			return 0; /* accept: part of contour */
+
+
+	return 1; /* refuse: would walk around a hole as a solid area */
+}
+
 RND_INLINE void pa_dic_emit_island_collect_from(pa_dic_ctx_t *ctx, pa_dic_isc_t *from)
 {
 	pa_dic_pt_box_relation_t ptst;
@@ -871,7 +899,15 @@ RND_INLINE void pa_dic_emit_island_collect_from(pa_dic_ctx_t *ctx, pa_dic_isc_t
 	if (from->vn == NULL)
 		return;
 
-	DEBUG_CLIP("     collect from: %ld;%ld\n", (long)from->x, (long)from->y);
+	/* do not start from a hole that is looping back to the starting point
+	   because we will end up mapping that hole as a contour; test case:
+	   dicer03 starting from 476;2316, to the left side hole in the first
+	   mapping attempt; hole that intersects the edge elsewhere should be
+	   accepted, see dicer04's little triangle right of the middle slicing plane */
+	if (pa_dic_is_hole_predict(ctx, from))
+		return;
+
+	DEBUG_CLIP("     collect from: ", Pcoord2(from->x, from->y), " clip box:", Pcoord2(ctx->clip.X1, ctx->clip.Y1), " .. ", Pcoord2(ctx->clip.X2, ctx->clip.Y2), "\n", 0);
 
 	/* Check where we can get from this intersection */
 	ptst = pa_dic_emit_island_predict(ctx, from->vn, from->pl);
@@ -889,19 +925,19 @@ RND_INLINE void pa_dic_emit_island_collect_from(pa_dic_ctx_t *ctx, pa_dic_isc_t
 	/* it's safe to start here, next deviation is going inside */
 	pa_dic_begin(ctx);
 	pa_dic_append(ctx, rnd_round(from->x), rnd_round(from->y));
-	DEBUG_CLIP("       append: %ld;%ld\n", (long)rnd_round(from->x), (long)rnd_round(from->y));
+	DEBUG_CLIP("       append: ", Pcoord2(from->x, from->y), "\n", 0);
 	from->pcollected = 1;
 
 	i = from;
 	do {
 		assert(i->vn != NULL); /* we need a pline intersection to start from */
 		vn = i->vn->next;
-		DEBUG_CLIP("      gather pline from: %ld;%ld (%ld;%ld -> %ld;%ld)\n", (long)i->x, (long)i->y, (long)vn->point[0], (long)vn->point[1], (long)vn->next->point[0], (long)vn->next->point[1]);
+		DEBUG_CLIP("      gather pline from: ", Pcoord2(i->x, i->y), " (", Pvnodep(vn), " -> ", Pvnodep(vn->next), ")\n", 0);
 		i = pa_dic_gather_pline(ctx, vn, i, from);
 		if (i == from)
 			break;
 		i->ecollected = 1;
-		DEBUG_CLIP("      gather edge from: %ld;%ld\n", (long)i->x, (long)i->y);
+		DEBUG_CLIP("      gather edge from: ", Pcoord2(i->x, i->y), "\n", 0);
 		i = pa_dic_gather_edge(ctx, i, from);
 		if (i == from)
 			break;
@@ -924,8 +960,8 @@ RND_INLINE void pa_dic_emit_island_common(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 RND_INLINE void pa_dic_emit_island_inverted(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 {
 	long num_pts = ctx->num_emits;
-	TODO("This is the same as the normal case... maybe just merge them");
-	DEBUG_CLIP("    emit island inverted\n");
+	TODO("This is the same as the normal case... maybe just merge them (pa_dic_emit_island_normal)");
+	DEBUG_CLIP("    emit island inverted\n", 0);
 	pa_dic_emit_island_common(ctx, pa);
 	if (num_pts == ctx->num_emits) {
 		/* Special case: technically there was an intersection but the walk-around
@@ -939,7 +975,7 @@ RND_INLINE void pa_dic_emit_island_inverted(pa_dic_ctx_t *ctx, rnd_polyarea_t *p
    drawing the contour of the island except for the box sections */
 RND_INLINE void pa_dic_emit_island_normal(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 {
-	DEBUG_CLIP("    emit island normal\n");
+	DEBUG_CLIP("    emit island normal\n", 0);
 	pa_dic_emit_island_common(ctx, pa);
 }
 
@@ -949,7 +985,7 @@ RND_INLINE void pa_dic_emit_island_expensive(pa_dic_ctx_t *ctx, rnd_polyarea_t *
 {
 	rnd_pline_t *pl;
 
-	DEBUG_CLIP("   emit island expensive\n");
+	DEBUG_CLIP("   emit island expensive\n", 0);
 	/* label all holes */
 	for(pl = pa->contours->next; pl != NULL; pl = pl->next) {
 		pa_dic_pline_label(ctx, pl);
@@ -980,7 +1016,7 @@ RND_INLINE void pa_dic_emit_island(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 	if ((ctx->clip.X1 >= pl->xmax) || (ctx->clip.Y1 >= pl->ymax)) return;
 	if ((ctx->clip.X2 <= pl->xmin) || (ctx->clip.Y2 <= pl->ymin)) return;
 
-	DEBUG_CLIP("  emit island: %ld;%ld\n", (long)pl->head->point[0], (long)pl->head->point[1]);
+	DEBUG_CLIP("  emit island: ", Pvnodep(pl->head), "\n", 0);
 	pa_dic_pline_label(ctx, pl);
 
 	/* first handle a few cheap common cases */
@@ -993,7 +1029,7 @@ RND_INLINE void pa_dic_emit_island(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 
 		case PA_PLD_INSIDE:
 			if (pl->next == NULL) {
-				/* there are no holes and the whole thig is inside -> plain emit */
+				/* there are no holes and the whole thing is inside -> plain emit */
 				pa_dic_emit_whole_pline(ctx, pl);
 				goto fin;
 			}
@@ -1064,6 +1100,9 @@ typedef struct pa_slc_ctx_s {
 	vtc0_t cuts;
 } pa_slc_ctx_t;
 
+#define PA_MIN(a,b)  ((a) < (b) ? (a) : (b))
+#define PA_MAX(a,b)  ((a) > (b) ? (a) : (b))
+
 /* map endpoints of the holes of a single contour */
 RND_INLINE void pa_slc_map_pline_holes(pa_slc_ctx_t *ctx, rnd_pline_t *contour)
 {
@@ -1076,10 +1115,10 @@ RND_INLINE void pa_slc_map_pline_holes(pa_slc_ctx_t *ctx, rnd_pline_t *contour)
 
 		ep = vtslc_alloc_append(&ctx->v, 2);
 		ep[0].pl = hole;
-		ep[0].x = RND_MAX(hole->xmin, ctx->minx);
+		ep[0].x = PA_MAX(hole->xmin, ctx->minx);
 		ep[0].side = 0;
 		ep[1].pl = hole;
-		ep[1].x = RND_MIN(hole->xmax, ctx->maxx);
+		ep[1].x = PA_MIN(hole->xmax, ctx->maxx);
 		ep[1].side = 1;
 		hole->flg.sliced = 0;
 	}
@@ -1130,9 +1169,9 @@ RND_INLINE void pa_slc_dump(pa_slc_ctx_t *ctx)
 	pa_slc_endp_t *ep;
 
 	for(n = 0, ep = ctx->v.array; n < ctx->v.used; n++,ep++)
-		DEBUG_SLICE(" %c @%d ^%d", ep->side==0 ? '{' : '}', ep->x, ep->height);
+		DEBUG_SLICE(ep->side==0 ? " { @" : " } @", Pint(ep->x), " ^", Pint(ep->height), 0);
 
-	DEBUG_SLICE("\n");
+	DEBUG_SLICE("\n", 0);
 #endif
 }
 
@@ -1152,7 +1191,7 @@ RND_INLINE void pa_slc_find_cuts(pa_slc_ctx_t *ctx)
 		long best_n, best_h = -1;
 		int removed;
 
-		DEBUG_SLICE(" slc [%d] ", remaining);
+		DEBUG_SLICE(" slc [", Plong(remaining), "] ", 0);
 		pa_slc_dump(ctx);
 
 		/* find highest tower to slice */
@@ -1171,7 +1210,7 @@ RND_INLINE void pa_slc_find_cuts(pa_slc_ctx_t *ctx)
 		xc = (x1+x2)/2;
 		vtc0_append(&ctx->cuts, xc);
 
-		DEBUG_SLICE("best: @%d ^%d cut at %d\n", best_n, best_h, xc);
+		DEBUG_SLICE("best: @", Plong(best_n), " ^", Plong(best_h), " cut at ", Pcoord(xc), "\n", 0);
 
 		removed = 0;
 		/* mark all affected plines already sliced and decrease heights and remaining */
@@ -1180,7 +1219,7 @@ RND_INLINE void pa_slc_find_cuts(pa_slc_ctx_t *ctx)
 				ep->pl->flg.sliced = 1;
 				removed = 1;
 				remaining--;
-				DEBUG_SLICE(" remove %d..%d\n", ep->pl->xmin, ep->pl->xmax);
+				DEBUG_SLICE(" remove ", Pcoord(ep->pl->xmin), "..", Pcoord(ep->pl->xmax), "\n", 0);
 				/* decrease height over this pline */
 				for(m = n, ep2 = ep; (m < ctx->v.used); m++,ep2++) {
 					if ((ep2->pl == ep->pl) && (ep2->x == ep->pl->xmax))
@@ -1194,14 +1233,14 @@ RND_INLINE void pa_slc_find_cuts(pa_slc_ctx_t *ctx)
 		   that's a bug */
 		assert(removed != 0);
 		if (removed == 0) {
-			rnd_message(RND_MSG_ERROR, "dicer: failed to cut hole. Please report this bug sending the board file.\n");
+			pa_error("dicer: failed to cut hole. Please report this bug sending the file that caused it.\n");
 			break;
 		}
 
 	}
 
 
-	DEBUG_SLICE("slc post ");
+	DEBUG_SLICE("slc post ", 0);
 	pa_slc_dump(ctx);
 
 	/* reset pline flags */
@@ -1267,8 +1306,7 @@ void rnd_polyarea_slice_noholes(pa_dic_ctx_t *ctx, rnd_polyarea_t *pa)
 }
 
 
-#if PA_USE_NEW_DICER
-/* Old, compatibility API */
+/*** Old, compatibility API ***/
 
 typedef struct {
 	/* user config: */
@@ -1343,4 +1381,4 @@ void rnd_polyarea_no_holes_dicer(rnd_polyarea_t *pa, rnd_coord_t clipX1, rnd_coo
 	pa_polyarea_free_all(&pa);
 	pa_dic_free_ctx(&ctx);
 }
-#endif
+
