@@ -119,7 +119,7 @@ rnd_pline_t *pa_pline_new(const rnd_vector_t pt)
 	if (pt != NULL) {
 		res->head->next = res->head->prev = res->head;
 		Vcpy2(res->head->point, pt);
-		pa_pline_box_bump(res, pt);
+		pa_pline_box_bump_pt(res, pt);
 	}
 
 	return res;
@@ -174,6 +174,23 @@ void pa_pline_optimize(rnd_pline_t *pl)
 	}
 }
 
+RND_INLINE double pa_vnode_curve_area(rnd_vnode_t *v)
+{
+	double a;
+
+	switch(v->flg.curve_type) {
+		case RND_VNODE_LINE: return 0;
+		case RND_VNODE_ARC:
+			a = pa_sect_area(
+				v->point[0], v->point[1],
+				v->next->point[0], v->next->point[1],
+				v->curve.arc.center[0], v->curve.arc.center[1],
+				v->curve.arc.adir);
+			return (v->curve.arc.adir == 0) ? -a : +a;
+	}
+	return 0;
+}
+
 void pa_pline_update(rnd_pline_t *pl, rnd_bool optimize)
 {
 	double area = 0;
@@ -191,9 +208,12 @@ void pa_pline_update(rnd_pline_t *pl, rnd_bool optimize)
 	p = (c = pl->head)->prev;
 	if (c != p) {
 		do {
-			/* calculate area for orientation */
-			area += ((double)p->point[0] - (double)c->point[0]) * ((double)p->point[1] + (double)c->point[1]);
-			pa_pline_box_bump(pl, c->point);
+			/* calculate area for orientation, including curve type extras */
+			area += ((double)p->point[0] - (double)c->point[0]) * ((double)p->point[1] + (double)c->point[1]) / 2.0;
+			area += pa_vnode_curve_area(p);
+
+			/* update bbox and corner count */
+			pa_pline_box_bump_curve(pl, c);
 			pl->Count++;
 		}
 		while ((c = (p = c)->next) != pl->head);
