@@ -39,11 +39,22 @@ rnd_vnode_t *rnd_pline_seg2vnode(void *box)
 /* Recalculate bbox of the segment from current coords */
 RND_INLINE void pa_seg_update_bbox(pa_seg_t *s)
 {
-	TODO("arc: the code below works only for lines");
-	s->box.X1 = pa_min(s->v->point[0], s->v->next->point[0]);
-	s->box.X2 = pa_max(s->v->point[0], s->v->next->point[0]) + 1;
-	s->box.Y1 = pa_min(s->v->point[1], s->v->next->point[1]);
-	s->box.Y2 = pa_max(s->v->point[1], s->v->next->point[1]) + 1;
+	switch(s->v->flg.curve_type) {
+		case RND_VNODE_LINE:
+			s->box.X1 = pa_min(s->v->point[0], s->v->next->point[0]);
+			s->box.X2 = pa_max(s->v->point[0], s->v->next->point[0]) + 1;
+			s->box.Y1 = pa_min(s->v->point[1], s->v->next->point[1]);
+			s->box.Y2 = pa_max(s->v->point[1], s->v->next->point[1]) + 1;
+			break;
+		case RND_VNODE_ARC:
+			{
+				rnd_rtree_box_t bb;
+				pb2_raw_arc_bbox(&bb, s->v->point, s->v->next->point, s->v->curve.arc.center, s->v->curve.arc.adir);
+				s->box.X1 = bb.x1; s->box.Y1 = bb.y1;
+				s->box.X2 = bb.x2 + 1; s->box.Y2 = bb.y2 + 1;
+			}
+			break;
+	}
 }
 
 void *rnd_poly_make_edge_tree(rnd_pline_t *pl)
@@ -130,50 +141,28 @@ RND_INLINE int man_dist_2(rnd_vector_t a, rnd_vector_t b)
 	return dx*dx + dy*dy;
 }
 
-
-RND_INLINE void pa_tweak_isc(rnd_vector_t v1a, rnd_vector_t v1b, rnd_vector_t v2a, rnd_vector_t v2b, rnd_vector_t isc)
+/* return if seg1 and seg2 overlap */
+RND_INLINE int pa_seg_seg_olap_(rnd_vnode_t *seg1, rnd_vnode_t *seg2)
 {
-#ifdef PA_TWEAK_ISC
-	/* See test case topo_pt_close.fawk: when an intersection point is very close
-	   to the endpoint of an edge, a very short edge would be created by edge
-	   splitting at isc. If there are multiple such very short edges
-	   nearby, that may change the topology. A cheap trick to avoid this is
-	   to round isc point to nearby edge endpoint if intersection is too close */
+	rnd_vector_t tmp1, tmp2;
 
+	if (seg1->flg.curve_type != seg2->flg.curve_type)
+		return 0; /* different curve shapes can not overlap */
 
-	/* special case: if two endpoints are close, move them together first, to
-	   avoid the corner case topo_pt_close2.fawk, where one side of the X
-	   crossing has original endpoints close but not incident so when the crossing
-	   is moved to one endpoint, we still end up with an 1 long edge */
-	if (man_dist_2(v1a, v2a) <= PA_TWEAK_ISC) Vcpy2(v1a, v2a);
-	if (man_dist_2(v1a, v2b) <= PA_TWEAK_ISC) Vcpy2(v1a, v2b);
-	if (man_dist_2(v1b, v2a) <= PA_TWEAK_ISC) Vcpy2(v1b, v2a);
-	if (man_dist_2(v1b, v2b) <= PA_TWEAK_ISC) Vcpy2(v1b, v2b);
-
-
-	if (man_dist_2(isc, v1a) <= PA_TWEAK_ISC) { Vcpy2(isc, v1a); return; }
-	if (man_dist_2(isc, v2a) <= PA_TWEAK_ISC) { Vcpy2(isc, v2a); return; }
-	if (man_dist_2(isc, v1b) <= PA_TWEAK_ISC) { Vcpy2(isc, v1b); return; }
-	if (man_dist_2(isc, v2b) <= PA_TWEAK_ISC) { Vcpy2(isc, v2b); return; }
-#endif
+	switch(seg1->flg.curve_type) {
+		case RND_VNODE_LINE:
+			return pa_vect_inters2(seg1->point, seg1->next->point, seg2->point, seg2->next->point, tmp1, tmp2, 1) == 2;
+		case RND_VNODE_ARC:
+			if (!Vequ2(seg1->curve.arc.center, seg2->curve.arc.center))
+				return 0;
+			return pb2_raw_isc_arc_arc(
+				seg1->point, seg1->next->point, seg1->curve.arc.center, seg1->curve.arc.adir,
+				seg2->point, seg2->next->point, seg2->curve.arc.center, seg2->curve.arc.adir,
+				tmp1, tmp2) == 2;
+	}
+	return 0;
 }
 
-RND_INLINE int pa_isc_edge_edge_(rnd_vnode_t *v1a, rnd_vnode_t *v1b, rnd_vnode_t *v2a, rnd_vnode_t *v2b, rnd_vector_t *isc1, rnd_vector_t *isc2)
-{
-	int res;
-	TODO("arc: this is where an arc-arc or line-arc or arc-line intersection would be detected then new point added");
-	res = pa_vect_inters2(v1a->point, v1b->point, v2a->point, v2b->point, *isc1, *isc2, 1);
-
-	if (res > 0) pa_tweak_isc(v1a->point, v1b->point, v2a->point, v2b->point, *isc1);
-	if (res > 1) pa_tweak_isc(v1a->point, v1b->point, v2a->point, v2b->point, *isc2);
-
-	return res;
-}
-
-int pa_isc_edge_edge(rnd_vnode_t *v1a, rnd_vnode_t *v1b, rnd_vnode_t *v2a, rnd_vnode_t *v2b, rnd_vector_t *isc1, rnd_vector_t *isc2)
-{
-	return pa_isc_edge_edge_(v1a, v1b, v2a, v2b, isc1, isc2);
-}
 
 /* Cancels the search and sets ctx->s if b is the same as ctx->v. Usefule
    for finding the segment for a node */
