@@ -146,10 +146,91 @@ rnd_bool_t rnd_is_point_in_convex_quad(rnd_vector_t p, rnd_vector_t *q)
 
 rnd_bool_t pa_is_node_on_line(rnd_vnode_t *node, rnd_vnode_t *l1, rnd_vnode_t *l2)
 {
-	TODO("Implement me");
-	return 0;
+	/* check if out of bbox */
+	if ((node->point[0] < l1->point[0]) && (node->point[0] < l2->point[0])) return 0;
+	if ((node->point[0] > l1->point[0]) && (node->point[0] > l2->point[0])) return 0;
+	if ((node->point[1] < l1->point[1]) && (node->point[1] < l2->point[1])) return 0;
+	if ((node->point[1] > l1->point[1]) && (node->point[1] > l2->point[1])) return 0;
+	if (Vequ2(node->point, l1->point) || Vequ2(node->point, l2->point)) return 0;
+
+	return (double)(node->point[0] - l2->point[0])*(double)(l2->point[1] - l1->point[1]) == (double)(l2->point[0] - l1->point[0])*(double)(node->point[1] - l2->point[1]);
 }
 
+rnd_bool_t pa_is_node_on_arc(rnd_vnode_t *node, rnd_vnode_t *arc)
+{
+	return pb2_raw_pt_on_arc(node->point, arc->point, arc->next->point, arc->curve.arc.center, arc->curve.arc.adir);
+}
+
+rnd_bool_t pa_is_node_on_curve(rnd_vnode_t *node, rnd_vnode_t *seg)
+{
+	switch(seg->flg.curve_type) {
+		case RND_VNODE_LINE: return pa_is_node_on_line(node, seg, seg->next);
+		case RND_VNODE_ARC: return pa_is_node_on_arc(node, seg);
+	}
+	return rnd_false;
+}
+
+RND_INLINE int pa_curve_isc_arc_line(rnd_vnode_t *arc, rnd_vnode_t *line, rnd_vector_t i1, rnd_vector_t i2)
+{
+	return pb2_raw_isc_arc_line(
+		arc->point, arc->next->point, arc->curve.arc.center, arc->curve.arc.adir,
+		line->point, line->next->point,
+		i1, i2);
+}
+
+RND_INLINE int pa_curve_isc_arc_arc(rnd_vnode_t *arc1, rnd_vnode_t *arc2, rnd_vector_t i1, rnd_vector_t i2)
+{
+	return pb2_raw_isc_arc_arc(
+		arc1->point, arc1->next->point, arc1->curve.arc.center, arc1->curve.arc.adir,
+		arc2->point, arc2->next->point, arc2->curve.arc.center, arc2->curve.arc.adir,
+		i1, i2);
+}
+
+RND_INLINE int pa_curves_isc(rnd_vnode_t *a1, rnd_vnode_t *a2, rnd_vector_t i1, rnd_vector_t i2)
+{
+	switch(a1->flg.curve_type) {
+		case RND_VNODE_LINE:
+			switch(a2->flg.curve_type) {
+				case RND_VNODE_LINE: return rnd_vect_inters2(a1->point, a1->next->point, a2->point, a2->next->point, i1, i2);
+				case RND_VNODE_ARC: return pa_curve_isc_arc_line(a2, a1, i1, i2);
+			}
+			break;
+
+		case RND_VNODE_ARC:
+			switch(a2->flg.curve_type) {
+				case RND_VNODE_LINE: return pa_curve_isc_arc_line(a1, a2, i1, i2);
+				case RND_VNODE_ARC: return pa_curve_isc_arc_arc(a1, a2, i1, i2);
+			}
+			break;
+	}
+
+	return 0; /* invalid curve type */
+}
+
+RND_INLINE void pa_curve_tangent(rnd_vector_t dst, rnd_vnode_t *vn, int which)
+{
+	rnd_vnode_t *cvn = (which < 0) ? vn->prev : vn; /* vn that provides curve fields */
+
+	switch(cvn->flg.curve_type) {
+		case RND_VNODE_LINE:
+			if (which < 0)
+				Vsub2(dst, vn->point, vn->prev->point);      /* vn to vn prev */
+			else
+				Vsub2(dst, vn->next->point, vn->point);      /* vn next to vn */
+			return;
+		case RND_VNODE_ARC:
+			pb2_raw_tangent_from_arc(dst, cvn->point, cvn->next->point, cvn->curve.arc.center, cvn->curve.arc.adir, vn->point);
+			if (which < 0) { /* preserve continous flow of directions, like for lines above */
+				dst[0] = -dst[0];
+				dst[1] = -dst[1];
+			}
+			return;
+	}
+	/* invalid curve type */
+	dst[0] = dst[1] = 0;
+}
+
+
 void pa_calc_angle_nn(pa_angle_t *dst, rnd_vector_t PT, rnd_vector_t OTHER)
 {
 	double dx, dy;
