@@ -44,11 +44,15 @@
 		(r)[1] = (a)[1] - (b)[1]; \
 	} while(0)
 
+RND_INLINE void pb2_arc_get_midpoint_dbl(pb2_seg_t *arc, double *mx, double *my);
+RND_INLINE void pb2_arc_shift_end_dbl(pb2_seg_t *arc, int end_idx, double offs, double *xout, double *yout);
+
+
 #include "pa_prints.h"
 #include "pb2_debug_txt.c"
 #include "pb2_debug_svg.c"
-#include "pb2_admin.c"
 #include "pb2_geo.c"
+#include "pb2_admin.c"
 
 #include "pb2_1.c"
 #include "pb2_2.c"
