@@ -72,12 +72,24 @@ struct pb2_seg_s {
 
 	union {
 		struct { /* circular arc */
-			rnd_vector_t center; /* part of the specification: saved */
+			/* fields part of the specification: saved */
+			rnd_vector_t center;
+			unsigned adir:1;  /* 0 means CCW around center which is also RND_PLF_INV, 1 means CW around center which is also RND_PLF_DIR (assuming SVG coord system: 0;0 on top left) */
+
 			/* cached fields (not saved) */
+#if 0
+			/* these are going to be used once we switch over to big coords */
 			rnd_ucoord_t r[6]; /* pa_big_coord_t really */
 			/* angle of point from the center [0..4); pa_big_coord_t aim = aim_int.aim_frac */
 			rnd_ucoord_t aim_frac[2]; /* fraction part of aim ("angle") for start and endpoint */
 			char aim_int[2]; /* integer part of the aim ("angle") for start and endpoint */
+#endif
+			double cx, cy; /* accurate center point */
+			double r;
+			double start, delta; /* angles in radian, in C and gengeo2d coord system */
+			double start_origc, delta_origc; /* same angles but from the original, integer centers */
+			double sect_area; /* signed area (negative if arc sector cuts away from the polyline area, bending inward) */
+			unsigned right_curving:1; /* set to 1 if arc has a point that's right of both endpoints; set during bbox calc */
 		} arc;
 	} shape;
 
@@ -97,7 +109,8 @@ struct pb2_cgout_s { /* outgoing edge from a node; not allocated separately, par
 	unsigned reverse:1;     /* when 1, end->start so this out is part of curve->out_end */
 	unsigned corner_mark:1; /* when 1, the corner/angle between this curve and the next is marked */
 	int nd_idx;             /* index of this out within cgnode->edges[] */
-	pa_angle_t angle;
+	pa_angle_t angle;       /* tangential angle of the segment going out from this node */
+	pa_angle_t angle2;      /* arc segments: angle in between tangential and radial, always pointing inward the seg */
 	PB2_UID
 };
 
@@ -208,10 +221,21 @@ extern int pb2_debug_dump_steps, pb2_debug_draw_steps;
 int pb2_exec(pb2_ctx_t *ctx, rnd_polyarea_t **res);
 
 /*** For filling up ctx with content ***/
-void pb2_1_map_seg_line(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, char poly_id, int isected);
+
+/* draw a straight line segment; order of p1 and p2 does not matter */
+void pb2_1_map_seg_line(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, char poly_id);
+
+/* draw a circular arc; order of p1 and p2 does matter; adir is the same as
+   adir in pb2_seg_s above */
+void pb2_1_map_seg_arc(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, const rnd_vector_t center, int adir, char poly_id);
 
 
 /*** For unit testing ***/
 extern int pb2_face_polarity_at_verbose;
 int pb2_face_polarity_at(pb2_ctx_t *ctx, rnd_vector_t pt, rnd_vector_t direction);
 
+#ifndef NDEBUG
+void pb2_draw_polyarea(pb2_ctx_t *ctx, FILE *F, const rnd_polyarea_t *pa, const char *clr, double fill_opacity);
+void pb2_svg_print_pline(FILE *F, const rnd_pline_t *pl);
+#endif
+
