@@ -29,6 +29,9 @@
 
 RND_INLINE int pb2_seg_nonzero(pb2_seg_t *seg)
 {
+	/* This works for arcs because arcs are split at vcrossings, so on top
+	   and bottom maximum/minimum so that remaining arcs don't have y
+	   inflexion from start to end */
 	if (seg->start[1] == seg->end[1])
 		return 0;
 	if (seg->start[1] > seg->end[1])
@@ -36,30 +39,29 @@ RND_INLINE int pb2_seg_nonzero(pb2_seg_t *seg)
 	return -1;
 }
 
-RND_INLINE void pb2_1_seg_bbox(pb2_seg_t *seg)
-{
-	seg->bbox.x1 = pa_min(seg->start[0], seg->end[0]);   seg->bbox.y1 = pa_min(seg->start[1], seg->end[1]);
-	seg->bbox.x2 = pa_max(seg->start[0], seg->end[0])+1; seg->bbox.y2 = pa_max(seg->start[1], seg->end[1])+1;
-}
-
+/* does not set shape, does not calculate bbox, does not register in rtree or ctx  */
 RND_INLINE pb2_seg_t *pb2_seg_new_(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2)
 {
 	pb2_seg_t *seg = calloc(sizeof(pb2_seg_t), 1);
 	PB2_UID_SET(seg);
 	memcpy(seg->start, p1, sizeof(rnd_vector_t));
 	memcpy(seg->end, p2, sizeof(rnd_vector_t));
-	pb2_1_seg_bbox(seg);
-	rnd_rtree_insert(&ctx->seg_tree, seg, &seg->bbox);
 	assert((p1[0] != p2[0]) || (p1[1] != p2[1])); /* 0 length seg is bad */
-	seg->next_all = ctx->all_segs;
-	ctx->all_segs = seg;
+	/* caller needs to fill in shape and call pb2_seg_new_bbox_and_reg_() */
 	return seg;
 }
 
-RND_INLINE pb2_seg_t *pb2_seg_new(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, char poly)
+RND_INLINE void pb2_seg_new_bbox_and_reg_(pb2_ctx_t *ctx, pb2_seg_t *seg)
 {
-	pb2_seg_t *seg = pb2_seg_new_(ctx, p1, p2);
+	pb2_seg_bbox(seg);
+	rnd_rtree_insert(&ctx->seg_tree, seg, &seg->bbox);
+	seg->next_all = ctx->all_segs;
+	ctx->all_segs = seg;
+}
 
+
+RND_INLINE pb2_seg_t *pb2_seg_new_common(pb2_ctx_t *ctx, pb2_seg_t *seg, char poly)
+{
 	switch(poly) {
 		case 'B':
 			seg->non0B = pb2_seg_nonzero(seg);
@@ -69,18 +71,54 @@ RND_INLINE pb2_seg_t *pb2_seg_new(pb2_ctx_t *ctx, const rnd_vector_t p1, const r
 			seg->non0A = pb2_seg_nonzero(seg);
 			break;
 	}
-
 	return seg;
 }
 
+RND_INLINE pb2_seg_t *pb2_seg_new_line(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, char poly)
+{
+	pb2_seg_t *seg = pb2_seg_new_(ctx, p1, p2);
+	pb2_seg_new_bbox_and_reg_(ctx, seg);
+	return pb2_seg_new_common(ctx, seg, poly);
+}
+
+RND_INLINE pb2_seg_t *pb2_seg_new_arc(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, const rnd_vector_t center, int adir, char poly)
+{
+	pb2_seg_t *seg = pb2_seg_new_(ctx, p1, p2);
+	seg->shape_type = RND_VNODE_ARC;
+	seg->shape.arc.center[0] = center[0];
+	seg->shape.arc.center[1] = center[1];
+	seg->shape.arc.adir = adir;
+	pb2_seg_arc_update_cache(ctx, seg);
+	pb2_seg_new_bbox_and_reg_(ctx, seg);
+	return pb2_seg_new_common(ctx, seg, poly);
+}
+
+RND_INLINE pb2_seg_t *pb2_new_seg_from(pb2_ctx_t *ctx, const pb2_seg_t *src, char poly_id)
+{
+	pb2_seg_t *seg = pb2_seg_new_(ctx, src->start, src->end);
+	seg->shape_type = src->shape_type;
+	memcpy(&seg->shape, &src->shape, sizeof(src->shape));
+	pb2_seg_new_common(ctx, seg, poly_id);
+	pb2_seg_new_bbox_and_reg_(ctx, seg);
+	return seg;
+}
+
+
 RND_INLINE pb2_seg_t *pb2_seg_new_alike(pb2_ctx_t *ctx, const rnd_vector_t p1, const rnd_vector_t p2, const pb2_seg_t *copy_from)
 {
 	pb2_seg_t *seg = pb2_seg_new_(ctx, p1, p2);
+	seg->shape_type = copy_from->shape_type;
+	memcpy(&seg->shape, &copy_from->shape, sizeof(copy_from->shape));
 	seg->cntA = copy_from->cntA;
 	seg->cntB = copy_from->cntB;
 	seg->non0A = copy_from->non0A;
 	seg->non0B = copy_from->non0B;
 	seg->discarded = copy_from->discarded;
+	seg->shape_type = copy_from->shape_type;
+	if (seg->shape_type > 0)
+		memcpy(&seg->shape, &copy_from->shape, sizeof(seg->shape));
+	pb2_seg_update_cache(ctx, seg);
+	pb2_seg_new_bbox_and_reg_(ctx, seg);
 	return seg;
 }
 
