@@ -75,9 +75,12 @@ struct rnd_vnode_s {
 
 	/* fields above are binary-compatible with the old poly lib */
 
+	/* A curve indicates that vnode to vnode->next is not a straight line */
 	union {
 		struct { /* circular arc */
-			rnd_vector_t center; /* part of the specification: saved */
+			/* fields part of the specification: saved */
+			rnd_vector_t center;
+			unsigned adir:1;  /* 0 means CCW around center which is also RND_PLF_INV, 1 means CW around center which is also RND_PLF_DIR (assuming SVG coord system: 0;0 on top left) */
 			/* cached fields (not saved) */
 			rnd_ucoord_t r[6]; /* pa_big_coord_t really */
 			/* angle of point from the center [0..4); pa_big_coord_t aim = aim_int.aim_frac */
