@@ -2,9 +2,9 @@
  *                            COPYRIGHT
  *
  *  libpolybool, 2D polygon bool operations
- *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
  *
- *  (Supported by NLnet NGI0 Entrust in 2023)
+ *  (Supported by NLnet NGI0 Entrust in 2023,2024)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -25,14 +25,6 @@
  *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
- *  This is a full rewrite of pcb-rnd's (and PCB's) polygon lib originally
- *  written by Harry Eaton in 2006, in turn building on "poly_Boolean: a
- *  polygon clip library" by Alexey Nikitin, Michael Leonov from 1997 and
- *  "nclip: a polygon clip library" Klamer Schutte from 1993.
- *
- *  English translation of the original paper the lib is largely based on:
- *  https://web.archive.org/web/20160418014630/http://www.complex-a5.ru/polyboolean/downloads/polybool_eng.pdf
- *
  */
 
 /* functions for generating polygons or polygon slices */
@@ -47,37 +39,49 @@
 
 #include <librnd/core/global_typedefs.h>
 #include "polyarea.h"
-#include <librnd/core/math_helper.h>
-#include <librnd/core/unit.h>
 
 #include "pa_math.c"
 
 #include "polygon1_gen.h"
 
-static double rotate_circle_seg[4];
+#define PA_M180                 (M_PI/180.0)
+static double rotate_circle_seg_pos[4], rotate_circle_seg_neg[4];
 int rotate_circle_seg_inited = 0;
 
 static void init_rotate_cache(void)
 {
 	if (!rotate_circle_seg_inited) {
-		double cos_ang = cos(2.0 * M_PI / RND_POLY_CIRC_SEGS_F);
-		double sin_ang = sin(2.0 * M_PI / RND_POLY_CIRC_SEGS_F);
+		double cos_ang, sin_ang;
+
+		cos_ang = cos(2.0 * M_PI / RND_POLY_CIRC_SEGS_F);
+		sin_ang = sin(2.0 * M_PI / RND_POLY_CIRC_SEGS_F);
+
+		rotate_circle_seg_pos[0] = cos_ang;
+		rotate_circle_seg_pos[1] = -sin_ang;
+		rotate_circle_seg_pos[2] = sin_ang;
+		rotate_circle_seg_pos[3] = cos_ang;
+
+		cos_ang = cos(-2.0 * M_PI / RND_POLY_CIRC_SEGS_F);
+		sin_ang = sin(-2.0 * M_PI / RND_POLY_CIRC_SEGS_F);
+
+		rotate_circle_seg_neg[0] = cos_ang;
+		rotate_circle_seg_neg[1] = -sin_ang;
+		rotate_circle_seg_neg[2] = sin_ang;
+		rotate_circle_seg_neg[3] = cos_ang;
 
-		rotate_circle_seg[0] = cos_ang;
-		rotate_circle_seg[1] = -sin_ang;
-		rotate_circle_seg[2] = sin_ang;
-		rotate_circle_seg[3] = cos_ang;
 		rotate_circle_seg_inited = 1;
 	}
 }
 
-static void rnd_poly_frac_circle_(rnd_pline_t * c, rnd_coord_t cx, rnd_coord_t cy, rnd_vector_t v, int range, int add_last)
+static void rnd_poly_frac_circle_(rnd_pline_t * c, rnd_coord_t cx, rnd_coord_t cy, rnd_vector_t v, int range, int add_last, rnd_poly_fct_t how)
 {
-	double ex, ey, rad1_x = (v[0]-cx), rad1_y = (v[1]-cy);
+	double ex, ey, rad1_x = (v[0]-cx), rad1_y = (v[1]-cy), *rotate_circle_seg;
 	int n, end;
 
 	init_rotate_cache();
 
+	rotate_circle_seg = (how & RND_POLY_FCT_REVERSE) ? rotate_circle_seg_neg : rotate_circle_seg_pos;
+
 	rnd_poly_vertex_include(c->head->prev, rnd_poly_node_create(v));
 
 	/* move vector to origin */
@@ -140,7 +144,7 @@ RND_INLINE int dquarter(rnd_vector_t v)
 
 /* Return 1 if v reached or webt beyond end, assuming CW direction in the
    rnd (screen) coord system */
-RND_INLINE int went_beyond_end(int start_dq, rnd_vector_t end, int end_dq, int rollover, rnd_vector_t v)
+RND_INLINE int went_beyond_end_pos(int start_dq, rnd_vector_t end, int end_dq, int rollover, rnd_vector_t v, int *was_in_same)
 {
 	int dq = dquarter(v);
 
@@ -155,7 +159,7 @@ RND_INLINE int went_beyond_end(int start_dq, rnd_vector_t end, int end_dq, int r
 		return 1;
 
 	if (dq == end_dq) { /* in the same dquarter */
-
+		*was_in_same = 1;
 		switch (dq % 8) {
 			/* axis aligned end vector */
 			case 0:
@@ -171,17 +175,67 @@ RND_INLINE int went_beyond_end(int start_dq, rnd_vector_t end, int end_dq, int r
 			case 7: if (v[1] >= end[1]) return 1; break;
 		}
 	}
+	else if (*was_in_same) {
+		/* if we were in the same quarter as end previously and left the quarter,
+		   we are surely beyond end; test case: pcb02 */
+		return 1;
+	}
 
 	return 0;
 }
 
-void rnd_poly_frac_circle_to(rnd_pline_t *c, rnd_vnode_t *insert_after, rnd_coord_t cx, rnd_coord_t cy, const rnd_vector_t start, const rnd_vector_t end)
+/* Return 1 if v reached or webt beyond end, assuming CCW direction in the
+   rnd (screen) coord system */
+RND_INLINE int went_beyond_end_neg(int start_dq, rnd_vector_t end, int end_dq, int rollover, rnd_vector_t v, int *was_in_same)
 {
-	double ex, ey, rad1_x, rad1_y;
-	int n, start_dq, end_dq, rollover;
+	int dq = dquarter(v);
+
+	if (rollover) {
+		if (dq < start_dq)
+			dq += 8;
+	}
+
+	if (dq > end_dq)
+		return 1;
+
+	if (dq == end_dq) { /* in the same dquarter */
+		*was_in_same = 1;
+		switch (dq % 8) {
+			/* axis aligned end vector */
+			case 0:
+			case 2:
+			case 4:
+			case 6:
+				return 1;
+
+			/* diagonal end vector */
+			case 1: if (v[1] <= end[1]) return 1; break;
+			case 3: if (v[1] >= end[1]) return 1; break;
+			case 5: if (v[1] >= end[1]) return 1; break;
+			case 7: if (v[1] <= end[1]) return 1; break;
+		}
+	}
+	else if (*was_in_same) {
+		/* if we were in the same quarter as end previously and left the quarter,
+		   we are surely beyond end; test case: pcb02 */
+		return 1;
+	}
+
+	return 0;
+}
+
+void rnd_poly_frac_circle_to(rnd_pline_t *c, rnd_vnode_t *insert_after, rnd_coord_t cx, rnd_coord_t cy, const rnd_vector_t start, const rnd_vector_t end, rnd_poly_fct_t how)
+{
+	double ex, ey, *rotate_circle_seg;
+	int n, start_dq, end_dq, rollover, was_in_same = 0;
 	rnd_vector_t rel_s, rel_e, v;
 	rnd_vnode_t *new_node;
 
+	if ((start[0] == end[0]) && (start[1] == end[1]))
+		return;
+
+	rotate_circle_seg = (how & RND_POLY_FCT_REVERSE) ? rotate_circle_seg_neg : rotate_circle_seg_pos;
+
 	rel_s[0] = start[0] - cx;
 	rel_s[1] = start[1] - cy;
 	rel_e[0] = end[0] - cx;
@@ -189,8 +243,6 @@ void rnd_poly_frac_circle_to(rnd_pline_t *c, rnd_vnode_t *insert_after, rnd_coor
 	start_dq = dquarter(rel_s);
 	end_dq = dquarter(rel_e);
 
-	rad1_x = rel_s[0]; rad1_y = rel_s[1];
-
 	/* degenerate case: zero length vector */
 	if (end_dq < 0)
 		return;
@@ -206,6 +258,15 @@ void rnd_poly_frac_circle_to(rnd_pline_t *c, rnd_vnode_t *insert_after, rnd_coor
 	v[0] = start[0]; v[1] = start[1];
 	ex = (v[0]-cx) * RND_POLY_CIRC_RADIUS_ADJ; ey = (v[1]-cy) * RND_POLY_CIRC_RADIUS_ADJ;
 
+	if (how & RND_POLY_FCT_SKIP_TINY) {
+		double dist2 = rnd_vect_dist2(rel_s, rel_e), mindist;
+		mindist = (ex + ey) * M_PI / RND_POLY_CIRC_SEGS_F;
+		if (dist2 < 4*mindist*mindist)
+			return;
+	}
+
+
+
 /*
 rnd_printf("frac circ at %mm;%mm: %mm;%mm %d .. %mm;%mm %d\n",
 	cx, cy, start[0], start[1], start_dq, end[0], end[1], end_dq);
@@ -224,8 +285,14 @@ rnd_printf("frac circ at %mm;%mm: %mm;%mm %d .. %mm;%mm %d\n",
 		er[1] = PA_ROUND(ey);
 
 		/* stop if went beyond end */
-		if (went_beyond_end(start_dq, rel_e, end_dq, rollover, er))
-			break;
+		if (how & RND_POLY_FCT_REVERSE) {
+			if (went_beyond_end_neg(start_dq, rel_e, end_dq, rollover, er, &was_in_same))
+				break;
+		}
+		else {
+			if (went_beyond_end_pos(start_dq, rel_e, end_dq, rollover, er, &was_in_same))
+				break;
+		}
 
 		v[0] = cx + er[0]; v[1] = cy + er[1];
 		new_node = rnd_poly_node_create(v);
@@ -237,12 +304,12 @@ rnd_printf("frac circ at %mm;%mm: %mm;%mm %d .. %mm;%mm %d\n",
 
 void rnd_poly_frac_circle(rnd_pline_t *c, rnd_coord_t cx, rnd_coord_t cy, rnd_vector_t v, int range)
 {
-	rnd_poly_frac_circle_(c, cx, cy, v, range, 0);
+	rnd_poly_frac_circle_(c, cx, cy, v, range, 0, 0);
 }
 
 void rnd_poly_frac_circle_end(rnd_pline_t *c, rnd_coord_t cx, rnd_coord_t cy, rnd_vector_t v, int range)
 {
-	rnd_poly_frac_circle_(c, cx, cy, v, range, 1);
+	rnd_poly_frac_circle_(c, cx, cy, v, range, 1, 0);
 }
 
 rnd_polyarea_t *rnd_poly_from_contour_nochk(rnd_pline_t *pl)
@@ -296,12 +363,21 @@ static void pa_poly_from_arc_stroke(rnd_pline_t *pl, rnd_coord_t cx, rnd_coord_t
 
 	for(n = 0; n < num_segs; n++, ang += da) {
 		rnd_vector_t v;
-		v[0] = cx - rx * cos(ang * RND_M180);
-		v[1] = cy + ry * sin(ang * RND_M180);
+		v[0] = cx - rx * cos(ang * PA_M180);
+		v[1] = cy + ry * sin(ang * PA_M180);
 		rnd_poly_vertex_include(pl->head->prev, rnd_poly_node_create(v));
 	}
 }
 
+#ifdef RND_API_VER
+#	include <librnd/core/unit.h>
+#	define LAST_POINT_FAR_ENOUGH RND_MM_TO_COORD(0.001)
+#else
+#	define LAST_POINT_FAR_ENOUGH 1000.0
+#endif
+
+#define PA_MAX(a,b)  ((a) > (b) ? (a) : (b))
+
 #define ARC_ANGLE 5
 static rnd_polyarea_t *pa_poly_from_arc_no_isc(rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t width, rnd_coord_t height, rnd_angle_t astart, rnd_angle_t adelta, rnd_coord_t thick, int end_caps)
 {
@@ -323,20 +399,20 @@ static rnd_polyarea_t *pa_poly_from_arc_no_isc(rnd_coord_t cx, rnd_coord_t cy, r
 	rnd_arc_get_endpt(cx, cy, width, height, astart, adelta, 1, &endx2, &endy2);
 
 	/* start with inner radius */
-	rx = MAX(width - half, 0);
-	ry = MAX(height - half, 0);
+	rx = PA_MAX(width - half, 0);
+	ry = PA_MAX(height - half, 0);
 
 	segs = 1;
 	if (thick > 0)
-		segs = MAX(segs, adelta * M_PI / 360 * sqrt(sqrt((double)rx * (double)rx + (double)ry * (double)ry) / RND_POLY_ARC_MAX_DEVIATION / 2 / thick));
-	segs = MAX(segs, adelta / ARC_ANGLE);
+		segs = PA_MAX(segs, adelta * M_PI / 360 * sqrt(sqrt((double)rx * (double)rx + (double)ry * (double)ry) / RND_POLY_ARC_MAX_DEVIATION / 2 / thick));
+	segs = PA_MAX(segs, adelta / ARC_ANGLE);
 
 	ang = astart;
 	da = (double)adelta / (double)segs;
 	radius_adj = (M_PI * da / 360) * (M_PI * da / 360) / 2;
 
-	v[0] = cx - rx * cos(ang * RND_M180);
-	v[1] = cy + ry * sin(ang * RND_M180);
+	v[0] = cx - rx * cos(ang * PA_M180);
+	v[1] = cy + ry * sin(ang * PA_M180);
 
 	pl = pa_pline_new(v);
 	if (pl == NULL)
@@ -346,8 +422,8 @@ static rnd_polyarea_t *pa_poly_from_arc_no_isc(rnd_coord_t cx, rnd_coord_t cy, r
 
 	/* find last point */
 	ang = astart + adelta;
-	v[0] = cx - rx * cos(ang * RND_M180) * (1 - radius_adj);
-	v[1] = cy + ry * sin(ang * RND_M180) * (1 - radius_adj);
+	v[0] = cx - rx * cos(ang * PA_M180) * (1 - radius_adj);
+	v[1] = cy + ry * sin(ang * PA_M180) * (1 - radius_adj);
 
 	/* add the round cap at the end */
 	if (end_caps)
@@ -361,13 +437,13 @@ static rnd_polyarea_t *pa_poly_from_arc_no_isc(rnd_coord_t cx, rnd_coord_t cy, r
 
 	/* explicitly draw the last point if the manhattan-distance is large enough */
 	ang = astart;
-	v2[0] = cx - rx * cos(ang * RND_M180) * (1 - radius_adj);
-	v2[1] = cy + ry * sin(ang * RND_M180) * (1 - radius_adj);
+	v2[0] = cx - rx * cos(ang * PA_M180) * (1 - radius_adj);
+	v2[1] = cy + ry * sin(ang * PA_M180) * (1 - radius_adj);
 	edx = (v[0] - v2[0]);
 	edy = (v[1] - v2[1]);
 	if (edx < 0) edx = -edx;
 	if (edy < 0) edy = -edy;
-	if (edx+edy > RND_MM_TO_COORD(0.001))
+	if (edx+edy > LAST_POINT_FAR_ENOUGH)
 		rnd_poly_vertex_include(pl->head->prev, rnd_poly_node_create(v2));
 
 	/* now add other round cap */
@@ -554,74 +630,3 @@ rnd_polyarea_t *rnd_poly_from_line(rnd_coord_t x1, rnd_coord_t y1, rnd_coord_t x
 	return rnd_poly_from_contour(pl);
 }
 
-/*** recursive polygon dicer ***/
-#if PA_USE_NEW_DICER == 0
-static void pa_polyarea_dicer_no_hole_1(rnd_polyarea_t *pa, void (*emit)(rnd_pline_t *, void *), void *user_data);
-
-static void pa_polyarea_dicer_recurse(rnd_polyarea_t *pa, void (*emit)(rnd_pline_t *, void *), void *user_data)
-{
-	rnd_polyarea_t *pan = pa, *next;
-
-	do {
-		next = pan->f;
-
-		/* remove pan from the list and recurse; when pan has no holes anymore, it gets freed */
-		pan->f = pan->b = pan;
-		pa_polyarea_dicer_no_hole_1(pan, emit, user_data);
-
-	} while((pan = next) != pa);
-}
-
-/* Recursively split up a single-island pa into no-hole contours and call
-   emit on each. Frees pa. */
-static void pa_polyarea_dicer_no_hole_1(rnd_polyarea_t *pa, void (*emit)(rnd_pline_t *, void *), void *user_data)
-{
-	rnd_pline_t *p = pa->contours;
-	rnd_polyarea_t *poly2, *left, *right;
-
-	assert((pa != NULL) && (pa->f == pa) && (pa->b == pa));
-
-	/* optimization: simpler no holes */
-	if (pa->contours->next == NULL) {
-		pa->contours = NULL; /* take over allocation ownership; not removed from rtree because rtree is destroyed soon */
-		emit(p, user_data);
-		pa_polyarea_free_all(&pa);
-		return;
-	}
-
-	/* make a rectangle of the left region slicing through the middle of the first hole */
-	poly2 = rnd_poly_from_rect(p->xmin, (p->next->xmin + p->next->xmax) / 2, p->ymin, p->ymax);
-	rnd_polyarea_and_subtract_free(pa, poly2, &left, &right);
-
-	if (left != NULL)
-		pa_polyarea_dicer_recurse(left, emit, user_data);
-
-	if (right != NULL)
-		pa_polyarea_dicer_recurse(right, emit, user_data);
-}
-
-void rnd_polyarea_no_holes_dicer(rnd_polyarea_t *pa, rnd_coord_t clipX1, rnd_coord_t clipY1, rnd_coord_t clipX2, rnd_coord_t clipY2, void (*emit)(rnd_pline_t *, void *), void *user_data)
-{
-	rnd_polyarea_t *pan, *next;
-
-	/* clip to the bounding box */
-	if ((clipX1 != clipX2) || (clipY1 != clipY2)) {
-		rnd_polyarea_t *clip_box = rnd_poly_from_rect(clipX1, clipX2, clipY1, clipY2);
-		rnd_polyarea_boolean_free(pa, clip_box, &pa, RND_PBO_ISECT);
-	}
-
-	if (pa == NULL)
-		return; /* happens easily when pa is outside of the clipping area */
-
-	/* iterate over the islands to call the recursive single-island dicer */
-	pan = pa;
-	do {
-		next = pan->f;
-
-		/* remove pan from the list and recurse; when pan has no holes anymore, it gets freed */
-		pan->f = pan->b = pan;
-		pa_polyarea_dicer_no_hole_1(pan, emit, user_data);
-	}
-	while((pan = next) != pa);
-}
-#endif
