@@ -131,20 +131,20 @@ big_local void big_free_temp(big_word *returned, big_word *temp_stack)
 
 
 /* word count: returns the number of words without leading zeros */
-big_local big_size big_len(big_word *a, big_size len)
+big_local big_size big_len(const big_word *a, big_size len)
 {
 	while (len && a[len-1] == 0) len--;
 	return len;
 }
 
 /* return whether a number is negative (assumung it's signed) */
-big_local int big_is_neg(big_word *a, big_size len)
+big_local int big_is_neg(const big_word *a, big_size len)
 {
 	return (a[len-1] >= BIG_NEG_BASE);
 }
 
 /* return whether a number is zero */
-big_local int big_is_zero(big_word *a, big_size len)
+big_local int big_is_zero(const big_word *a, big_size len)
 {
 	big_size i;
 	for(i = 0; i < len; i++)
@@ -154,7 +154,7 @@ big_local int big_is_zero(big_word *a, big_size len)
 }
 
 /* Returns sign: -1 for negative, 0 for zero and +1 for positiver */
-big_local int big_sgn(big_word *a, big_size len)
+big_local int big_sgn(const big_word *a, big_size len)
 {
 	if (big_is_neg(a, len))
 		return -1;
@@ -165,7 +165,7 @@ big_local int big_sgn(big_word *a, big_size len)
 
 
 /* cmp a,b with same len (top word set); unsigned */
-big_local int big_cmpn(big_word *a, big_word *b, big_size len)
+big_local int big_cmpn(const big_word *a, const big_word *b, big_size len)
 {
 	while (len--)
 		if (a[len] != b[len])
@@ -174,7 +174,7 @@ big_local int big_cmpn(big_word *a, big_word *b, big_size len)
 }
 
 /* cmp a,b with different len; unsigned */
-big_local int big_cmp(big_word *a, big_size alen, big_word *b, big_size blen)
+big_local int big_cmp(const big_word *a, big_size alen, const big_word *b, big_size blen)
 {
 	alen = big_len(a, alen);
 	blen = big_len(b, blen);
@@ -184,7 +184,7 @@ big_local int big_cmp(big_word *a, big_size alen, big_word *b, big_size blen)
 }
 
 /* same as cmpn but assumes signed numbers */
-big_local int big_signed_cmpn(big_word *a, big_word *b, big_size len)
+big_local int big_signed_cmpn(const big_word *a, const big_word *b, big_size len)
 {
 	int sa = big_sgn(a, len), sb = big_sgn(b, len);
 
@@ -235,7 +235,7 @@ big_local big_word big_sub111(big_word *r, big_word x, big_word y, big_word z)
 }
 
 /* r[] = x[] + y, return carry, r maybe same as x */
-big_local big_word big_add1(big_word *r, big_word *x, big_size len, big_word y)
+big_local big_word big_add1(big_word *r, const big_word *x, big_size len, big_word y)
 {
 	big_size i;
 	for (i = 0; i < len; i++)
@@ -244,7 +244,7 @@ big_local big_word big_add1(big_word *r, big_word *x, big_size len, big_word y)
 }
 
 /* r[] = x[] - y, return carry, r maybe same as x */
-big_local big_word big_sub1(big_word *r, big_word *x, big_size len, big_word y)
+big_local big_word big_sub1(big_word *r, const big_word *x, big_size len, big_word y)
 {
 	big_size i;
 	for (i = 0; i < len; i++)
@@ -253,7 +253,7 @@ big_local big_word big_sub1(big_word *r, big_word *x, big_size len, big_word y)
 }
 
 /* r[] = x[] + y[] + c, return carry, r maybe same as x and y */
-big_local big_word big_addn(big_word *r, big_word *x, big_word *y, big_size len, big_word c)
+big_local big_word big_addn(big_word *r, const big_word *x, const big_word *y, big_size len, big_word c)
 {
 	big_size i;
 	for (i = 0; i < len; i++)
@@ -262,7 +262,7 @@ big_local big_word big_addn(big_word *r, big_word *x, big_word *y, big_size len,
 }
 
 /* r[] = x[] - y[] - c, return carry, r maybe same as x and y */
-big_local big_word big_subn(big_word *r, big_word *x, big_word *y, big_size len, big_word c)
+big_local big_word big_subn(big_word *r, const big_word *x, const big_word *y, big_size len, big_word c)
 {
 	big_size i;
 	for (i = 0; i < len; i++)
@@ -271,7 +271,7 @@ big_local big_word big_subn(big_word *r, big_word *x, big_word *y, big_size len,
 }
 
 /* a = -b, a maybe same as b */
-big_local big_word big_neg(big_word *a, big_word *b, big_size len)
+big_local big_word big_neg(big_word *a, const big_word *b, big_size len)
 {
 	big_size i;
 	big_word carry = 0;
@@ -311,7 +311,7 @@ big_local big_word big_addmul11(big_word *lo, big_word a, big_word b)
 }
 
 /* a += b*c; only fill a[0..len-1], return a[len] */
-big_local big_word big_iaddmul1(big_word *a, big_word *b, big_size len, big_word c)
+big_local big_word big_iaddmul1(big_word *a, const big_word *b, big_size len, big_word c)
 {
 	big_size i;
 	big_word t=0;
@@ -324,7 +324,7 @@ big_local big_word big_iaddmul1(big_word *a, big_word *b, big_size len, big_word
 }
 
 /* a = b*c+d, a may alias b, return a[len] (overflow word) */
-big_local big_word big_mul1(big_word *a, big_word *b, big_size len, big_word c, big_word d)
+big_local big_word big_mul1(big_word *a, const big_word *b, big_size len, big_word c, big_word d)
 {
 	big_word hi, lo = d;
 	big_size i;
@@ -337,7 +337,7 @@ big_local big_word big_mul1(big_word *a, big_word *b, big_size len, big_word c,
 }
 
 /* a = b*c where a!=b, a!=c, alen>=len, faster if b>c, returns overflow */
-big_local int big_mul(big_word *a, big_size alen, big_word *b, big_word *c, big_size len)
+big_local int big_mul(big_word *a, big_size alen, const big_word *b, const big_word *c, big_size len)
 {
 	big_size i, n, blen=len, clen=len;
 	big_word t;
@@ -366,7 +366,7 @@ big_local int big_mul(big_word *a, big_size alen, big_word *b, big_word *c, big_
 	return oflow;
 }
 
-big_local int big_signed_mul(big_word *a_, big_size alen, big_word *b_, big_word *c_, big_size len)
+big_local int big_signed_mul(big_word *a_, big_size alen, const big_word *b_, const big_word *c_, big_size len)
 {
 	big_word *a, atmp[BIG_STACK_TEMP_LEN], *b, btmp[BIG_STACK_TEMP_LEN], *c, ctmp[BIG_STACK_TEMP_LEN];
 	int res, nega = 0, freeb = 0, freec = 0;
@@ -381,7 +381,7 @@ big_local int big_signed_mul(big_word *a_, big_size alen, big_word *b_, big_word
 		nega = 1;
 	}
 	else
-		b = b_;
+		b = (big_word *)b_;
 
 	if (big_is_neg(c_, len)) {
 		c = BIG_ALLOC_TEMP(ctmp, len);
@@ -390,7 +390,7 @@ big_local int big_signed_mul(big_word *a_, big_size alen, big_word *b_, big_word
 		nega = !nega;
 	}
 	else
-		c = c_;
+		c = (big_word *)c_;
 
 	if (nega) {
 		a = BIG_ALLOC_TEMP(atmp, alen);
@@ -480,7 +480,7 @@ big_local big_word big_div11(big_word *q, big_word hi, big_word lo, big_word d)
 
 /* a = b<<shift, where shift<BIG_BITS, len(a)=len, return bits shifted out,
    a maybe same as b */
-big_local big_word big_shl1(big_word *a, big_word *b, big_size len, big_size shift)
+big_local big_word big_shl1(big_word *a, const big_word *b, big_size len, big_size shift)
 {
 	big_word r;
 	if (len == 0 || shift == 0)
@@ -494,7 +494,7 @@ big_local big_word big_shl1(big_word *a, big_word *b, big_size len, big_size shi
 
 /* a = b>>shift, where shift<BIG_BITS, len(a)=len, return bits shifted out,
    a maybe same as b */
-big_local big_word big_shr1(big_word *a, big_word *b, big_size len, unsigned shift)
+big_local big_word big_shr1(big_word *a, const big_word *b, big_size len, unsigned shift)
 {
 	big_size i;
 	big_word r;
@@ -508,7 +508,7 @@ big_local big_word big_shr1(big_word *a, big_word *b, big_size len, unsigned shi
 }
 
 /* a = b/c, c!=0, len(a)=len, return r so a*c+r == b */
-big_local big_word big_div1_(big_word *a, big_word *b, big_size len, big_word c)
+big_local big_word big_div1_(big_word *a, const big_word *b, big_size len, big_word c)
 {
 	big_word r = 0;
 	big_size i = len;
@@ -525,7 +525,7 @@ big_local int big_less2(big_word x1, big_word x0, big_word y1, big_word y0)
 
 /* Compute u/v as q*v+r = u, where len(tmp) >= 2*vlen+ulen+2,
    vlen >= 2, ulen >= vlen, len(q) = ulen-vlen+1, len(r) = vlen */
-big_local void big_div_(big_word *q, big_word *r, big_word *u, big_size ulen, big_word *v, big_size vlen, big_word *tmp)
+big_local void big_div_(big_word *q, big_word *r, big_word *u, big_size ulen, const big_word *v, big_size vlen, big_word *tmp)
 {
 	big_word *qhv;
 	big_size j;
@@ -619,7 +619,7 @@ big_local big_word big_div1(big_word *a, big_word *b, big_size len, big_word c)
    len(q) = ulen-vlen+1
    len(r) = vlen
  */
-big_local int big_div(big_word *q, big_word *r, big_word *u, big_size ulen, big_word *v, big_size vlen, big_word *tmp)
+big_local int big_div(big_word *q, big_word *r, big_word *u, big_size ulen, const big_word *v, big_size vlen, big_word *tmp)
 {
 	big_word tmp_stack[BIG_STACK_TEMP_LEN];
 	int n;
@@ -651,7 +651,7 @@ big_local int big_div(big_word *q, big_word *r, big_word *u, big_size ulen, big_
 	return 0;
 }
 
-big_local int big_signed_div(big_word *q_, big_word *r_, big_word *u_, big_size ulen, big_word *v_, big_size vlen, big_word *tmp)
+big_local int big_signed_div(big_word *q_, big_word *r_, const big_word *u_, big_size ulen, const big_word *v_, big_size vlen, big_word *tmp)
 {
 	big_word *u, utmp[BIG_STACK_TEMP_LEN], *v, vtmp[BIG_STACK_TEMP_LEN];
 	big_word *q, qtmp[BIG_STACK_TEMP_LEN], *r, rtmp[BIG_STACK_TEMP_LEN];
@@ -667,7 +667,7 @@ big_local int big_signed_div(big_word *q_, big_word *r_, big_word *u_, big_size
 		negr = 1;
 	}
 	else
-		u = u_;
+		u = (big_word *)u_;
 
 	if (big_is_neg(v_, vlen)) {
 		v = BIG_ALLOC_TEMP(vtmp, vlen);
@@ -676,7 +676,7 @@ big_local int big_signed_div(big_word *q_, big_word *r_, big_word *u_, big_size
 		negq = !negq;
 	}
 	else
-		v = v_;
+		v = (big_word *)v_;
 
 	if (negq) {
 		q = BIG_ALLOC_TEMP(qtmp, qlen);
@@ -814,7 +814,7 @@ big_local big_size big_signed_fromstr(big_word *a, big_size len, char *s, char *
 
 
 /* convert a to str, dlen is base: dlen <= 256 && dlen >= 2 */
-big_local big_size big_tostr(char *s, big_size slen, big_word *a, big_size alen, char *digits, int dlen)
+big_local big_size big_tostr(char *s, big_size slen, const big_word *a, big_size alen, const char *digits, int dlen)
 {
 	big_size basebits, i, j;
 
@@ -906,7 +906,7 @@ end:
 
 
 
-big_local big_size big_signed_tostr(char *s, big_size slen, big_word *a, big_size alen, char *digits, int dlen)
+big_local big_size big_signed_tostr(char *s, big_size slen, const big_word *a, big_size alen, const char *digits, int dlen)
 {
 	big_word *tmp, tmp_[BIG_STACK_TEMP_LEN];
 	big_size res;
