@@ -1,14 +1,14 @@
 #define RND_APP_PREFIX(x)  rndtest_ ## x
 
 #include <librnd/core/unit.h>
-#include <librnd/core/hid_init.h>
-#include <librnd/core/hid.h>
+#include <librnd/hid/hid_init.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/buildin.hidlib.h>
 #include <librnd/core/conf.h>
-#include <librnd/core/buildin.hidlib.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/plugins.h>
 #include <librnd/core/hidlib.h>
-#include <librnd/core/hidlib_conf.h>
+#include <librnd/core/rnd_conf.h>
 #include <librnd/poly/polyarea.h>
 
 #include "glue.c"
@@ -32,13 +32,13 @@ int main(int argc, char *argv[])
 
 	rnd_fix_locale_and_env();
 
-	rnd_plugin_add_dir("include/usr/lib/librnd3/plugins");
+	rnd_plugin_add_dir("include/usr/lib/librnd4/plugins");
 	rnd_conf_force_set_bool(rnd_conf.rc.dup_log_to_stderr, 1);
 
 	rnd_main_args_init(&ga, argc, action_args);
 
 
-	rnd_hidlib_init1(conf_core_init);
+	rnd_hidlib_init1(conf_core_init, ".");
 	for(n = 1; n < argc; n++)
 		n += rnd_main_args_add(&ga, argv[n], argv[n+1]);
 
@@ -64,7 +64,7 @@ int main(int argc, char *argv[])
 		exit(n);
 	}
 
-	if (rnd_main_exported(&ga, &CTX.hidlib, 0)) {
+	if (rnd_main_exported(&ga, &CTX.hidlib, 0, NULL)) {
 		fprintf(stderr, "main_exported fail\n");
 		rnd_main_args_uninit(&ga);
 		exit(1);
