@@ -3,6 +3,12 @@
 # Read and parse all pups, make an array of file formats of
 # FMTS[import|export, type] then read stdin and execute the awk
 # script specified in $1 on it
+#
+# Can be extended with awk commands from global variables:
+#  $type_detect_begin - should append to types
+#  $type_detect_awk - should override type
+#
+
 awk_on_formats()
 {
 (for n in $proot/*/*.pup
@@ -17,6 +23,7 @@ cat
 BEGIN {
 	osep = " <br> "
 	types="board footprint netlist image misc"
+'"$type_detect_begin"'
 	pends = 0
 	disabled = 0
 }
@@ -64,8 +71,12 @@ function add(name, dir   ,type,lname)
 		type = "image"
 	else
 		type = "misc"
+
 # DO NOT FORGET TO UDPATE types IN BEGIN ^^^
 
+'"$type_detect_awk"'
+
+
 	if (FMTS[dir, type] == "")
 		FMTS[dir, type] = name
 	else
@@ -84,7 +95,7 @@ function flush_pends(    n)
 	disabled = 0
 }
 
-(mode == 1) && ($1 == "$fmt-feature-r") {
+(mode == 1) && (($1 == "$fmt-feature-r") || ($1 == "$fmt-feature-i")) {
 	$1=""
 	PENDING1[pends] = $0
 	PENDING2[pends] = "import"
@@ -92,7 +103,7 @@ function flush_pends(    n)
 	next
 }
 
-(mode == 1) && ($1 == "$fmt-feature-w") {
+(mode == 1) && (($1 == "$fmt-feature-w") || ($1 == "$fmt-feature-e")) {
 	$1=""
 	PENDING1[pends] = $0
 	PENDING2[pends] = "export"
